<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2018 Amasty (https://www.amasty.com)
 * @package Amasty_SeoGoogleSitemap
 */


class Amasty_SeoGoogleSitemap_Block_Adminhtml_Sitemap_Edit_Tab_Products extends Mage_Adminhtml_Block_Widget_Form
{
    protected function _prepareForm()
    {
        $form = new Varien_Data_Form();
        $this->setForm($form);
        $model = Mage::registry('am_sitemap_profile');
        $this->_generateProductFields();
        $this->getForm()->setValues($model->getData());

        $this->_generateDependencies();

        return parent::_prepareForm();
    }


    protected function _generateDependencies()
    {
        $this->setChild(
            'form_after',
            $this->getLayout()
                ->createBlock('adminhtml/widget_form_element_dependence')
                ->addFieldMap('products', 'products')
                ->addFieldMap('prod_out_of_stock', 'prod_out_of_stock')
                ->addFieldMap('hreflang_prod', 'hreflang_prod')
                ->addFieldMap('products_thumbs', 'products_thumbs')
                ->addFieldMap('products_captions_template', 'products_captions_template')
                ->addFieldMap('products_captions', 'products_captions')
                ->addFieldMap('products_captions_all', 'products_captions_all')
                ->addFieldMap('products_priority', 'products_priority')
                ->addFieldMap('products_frequency', 'products_frequency')
                ->addFieldMap('products_modified', 'products_modified')
                ->addFieldMap('batch_size_prod', 'batch_size_prod')
                ->addFieldDependence('products_captions_template', 'products', 1)
                ->addFieldDependence('prod_out_of_stock', 'products', 1)
                ->addFieldDependence('hreflang_prod', 'products', 1)
                ->addFieldDependence('products_priority', 'products', 1)
                ->addFieldDependence('products_frequency', 'products', 1)
                ->addFieldDependence('products_modified', 'products', 1)
                ->addFieldDependence('batch_size_prod', 'products', 1)
                ->addFieldDependence('products_thumbs', 'products', 1)
                ->addFieldDependence('products_captions', 'products', 1)
                ->addFieldDependence('products_captions_all', 'products', 1)
                ->addFieldDependence('products_captions_template', 'products_thumbs', 1)
                ->addFieldDependence('products_captions_template', 'products_captions', 1)
                ->addFieldDependence('products_captions', 'products_thumbs', 1)
        );
    }

    protected function _generateProductFields()
    {
        /** @var Amasty_SeoGoogleSitemap_Helper_Data $helper */
        $helper = Mage::helper('amseogooglesitemap');

        $fieldset = $this->getForm()->addFieldset(
            'amseogooglesitemap_form_content_products',
            array('legend' => $helper->__('Products'))
        );

        $fieldset->addField(
            'products',
            'select',
            array(
                'label' => $helper->__('Include products'),
                'name' => 'products',
                'title' => $helper->__('Include products'),
                'options' => $helper->getYesNo()
            )
        );

        $fieldset->addField(
            'prod_out_of_stock',
            'select',
            array(
                'label' => $helper->__('Include Out of Stock Products'),
                'name' => 'prod_out_of_stock',
                'title' => $helper->__('Include Out of Stock Products'),
                'options' => array(
                    Amasty_SeoGoogleSitemap_Model_Sitemap::DISABLE_OUT_OF_STOCK => $this->__('No'),
                    Amasty_SeoGoogleSitemap_Model_Sitemap::ENABLE_OUT_OF_STOCK => $this->__('Yes')
                )
            )
        );

        $fieldset->addField(
            'hreflang_prod',
            'select',
            array(
                'label' => $helper->__('Add Hreflang'),
                'name' => 'hreflang_prod',
                'title' => $helper->__('Add Hreflang'),
                'options' => $helper->getYesNo()
            )
        );

        $fieldset->addField(
            'products_thumbs',
            'select',
            array(
                'label' => $helper->__('Add Images'),
                'name' => 'products_thumbs',
                'title' => $helper->__('Add Images'),
                'options' => $helper->getYesNo()
            )
        );

        $fieldset->addField(
            'products_captions',
            'select',
            array(
                'label' => $helper->__('Add Images Titles'),
                'name' => 'products_captions',
                'title' => $helper->__('Add Images Titles'),
                'options' => $helper->getYesNo()
            )
        );

        $fieldset->addField(
            'products_captions_all',
            'select',
            array(
                'label' => $helper->__('Use All Product Images'),
                'name' => 'products_captions_all',
                'title' => $helper->__('Use All Product Images'),
                'options' => $helper->getYesNo()
            )
        );

        $note = $helper->__(
            'Specify text to be used for empty captions with {product_name} placeholder for product name. 
        Example - "enjoy {product_name} from e-store"'
        );
        $fieldset->addField(
            'products_captions_template',
            'text',
            array(
                'label' => $helper->__('Template for image title'),
                'name' => 'products_captions_template',
                'title' => $helper->__('Template for image title'),
                'note' => $note
            )
        );

        $fieldset->addField(
            'products_priority',
            'text',
            array(
                'label' => $helper->__('Priority'),
                'name' => 'products_priority',
                'note' => '0.0-1.0',
                'class' => 'validate-number validate-number-range number-range-0.0-1.0'
            )
        );

        $fieldset->addField(
            'products_frequency',
            'select',
            array(
                'label' => $helper->__('Frequency'),
                'name' => 'products_frequency',
                'title' => $helper->__('Frequency'),
                'options' => $helper->getFrequency()
            )
        );

        $fieldset->addField(
            'products_modified',
            'select',
            array(
                'label' => $helper->__('Include Last Modified'),
                'name' => 'products_modified',
                'title' => $helper->__('Include Last Modified'),
                'options' => $helper->getYesNo()
            )
        );

        $fieldset->addField(
            'batch_size_prod',
            'text',
            array(
                'label' => $helper->__('Batch Size'),
                'name' => 'batch_size_prod',
                'title' => $helper->__('Batch Size'),
                'note' => $helper->__('Indicate how many items will be handled during one iteration.')
            )
        );
    }
}
