<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2018 Amasty (https://www.amasty.com)
 * @package Amasty_SeoGoogleSitemap
 */


class Amasty_SeoGoogleSitemap_Model_Observer
{
    const LOG_FILE_NAME = 'amseogooglesitemap.log';
    const MAX_ITERATOR_COUNT = 100000;

    public function generate()
    {
        $appEmulation = Mage::getSingleton('core/app_emulation');
        //Start environment emulation of the specified store
        $initialEnvironmentInfo = $appEmulation->startEnvironmentEmulation(0);

        $profiles = Mage::getModel('amseogooglesitemap/sitemap')->getResourceCollection();
        foreach ($profiles as $profile) {
            $this->generateXmlByCron($profile);
        }

        $appEmulation->stopEnvironmentEmulation($initialEnvironmentInfo);
        return true;
    }

    protected function generateXmlByCron(Amasty_SeoGoogleSitemap_Model_Sitemap $profile)
    {
        $profile->clearSession();
        $this->logData($this->getAmastyHelper()->__('Starting generation for Sitemap #%s', $profile->getId()));

        $iterator = 0;
        while ($iterator++ < self::MAX_ITERATOR_COUNT) {
            try {
                $profile->run();
                $logs = $profile->getLogs();

                if ($logs && is_array($logs)) {
                    foreach ($logs as $message) {
                        $this->logData($message);
                    }
                    $profile->clearLogs();
                } else {
                    Mage::throwException($this->getAmastyHelper()->__('Generation was stopped. Empty result.'));
                }
            } catch (Exception $exception) {
                $this->logData($exception->getMessage());
                break;
            }
        }
    }

    /**
     * @return Amasty_SeoGoogleSitemap_Helper_Data
     */
    protected function getAmastyHelper()
    {
        return Mage::helper('amseogooglesitemap');
    }

    protected function logData($message)
    {
        if (Mage::getStoreConfig('amseogooglesitemap/amxmlsitemap_cron/log')) {
            Mage::log($message, null, self::LOG_FILE_NAME);
        }
    }
}
