<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2018 Amasty (https://www.amasty.com)
 * @package Amasty_SeoGoogleSitemap
 */


class Amasty_SeoGoogleSitemap_Model_Sitemap extends Amasty_SeoGoogleSitemap_Model_SitemapAbstract
{
    const SHOP_BY_BRANDS = 'shop_by_brands';
    const NAVIGATION = 'navigation';
    const DEFAULT_SIZE_CAT = 30;
    const DEFAULT_SIZE_PROD = 100;
    const DEFAULT_SIZE_PAGE = 30;
    const DEFAULT_SIZE_TAG = 30;
    const DISABLE_OUT_OF_STOCK = 2;
    const ENABLE_OUT_OF_STOCK = 1;

    /**
     * Generate XML file
     *
     * @return Mage_Sitemap_Model_Sitemap
     */
    public function run()
    {
        $this->initializeGeneration();

        $step = $this->getCurrentStep();
        if ($step == 1) {
            $this->_iterator = 0;
            $this->flushSeoSingleUrlCache();
            $this->removeTmpFiles();
        }

        $this->generateSiteStepData();
        $this->getSession()->setData(self::CURRENT_STEP, ++$step);

        if ($this->isAllGenerationDone()) {
            $this->splitFileDependOnSize();
            $this->setLastRun(Mage::getSingleton('core/date')->gmtDate('Y-m-d H:i:s'));
            $this->save();
            $this->_removeOldFiles();
            Mage::throwException($this->getAmastyHelper()->__('Site Map Generation was finished'));
        }

        Mage::helper('amseogooglesitemap/hrefurl')->clean();
    }

    protected function generateSiteStepData()
    {
        $currentSection = $this->getCurrentSection();

        if (isset($this->_generationSteps[$currentSection])) {
            $methodName = $this->generateMethodName($this->_generationSteps[$currentSection]);

            if (method_exists($this, $methodName)) {
                Mage::app()->setCurrentStore($this->_storeId);
                $data = call_user_func(array($this, $methodName));
                Mage::app()->setCurrentStore(0);
                if ($data) {
                    $this->writeDataToFile($data);
                }
            } else {
                $this->addResponseMessage($this->getAmastyHelper()->__('All site data was generated'));
                $this->finishSection(false);
            }

        } else {
            $this->getSession()->setData(self::GENERATION_DONE, true);
        }
    }

    protected function writeDataToFile($data)
    {
        $path = $this->getPath();
        $name = $this->getSitemapFileName();
        $name = str_replace('.xml', '_tmp.xml', $name);

        $io = new Varien_Io_File();
        $io->setAllowCreateFolders(true);
        $io->open(array('path' => $path));
        $io->streamOpen($name, 'a');

        $dataCount = count($data);
        for ($i = 0; $i < $dataCount; $i++) {
            if (false !== strpos($data[$i], '<loc></loc>')) {
                continue;
            }

            $io->streamWrite($data[$i] . "\n");
        }

        $io->streamClose();
    }

    protected function splitFileDependOnSize()
    {
        /* read tmp file and get all items*/
        $path = $this->getPath();
        $name = $this->getSitemapFileName();
        $tmpName = str_replace('.xml', '_tmp.xml', $name);

        $io = new Varien_Io_File();
        $io->open(array('path' => $path));
        $content = $io->read($tmpName);
        $this->_xml = explode(PHP_EOL, $content);

        $pieces = array();
        $this->_iterator = 0;
        $isChunk = $this->getMaxItems() > 0 && count($this->_xml) > $this->getMaxItems();
        $isOneFile = false;
        if ($this->getMaxItems() == 0 && $this->getMaxFileSize() == 0) {
            $isOneFile = true;
        }
        if ($isChunk) {
            $split = array_chunk($this->_xml, $this->getMaxItems(), false);
            foreach ($split as $chunk) {
                $pieces = array_merge($pieces, $this->_writePortion($chunk, false));
            }
        } else {
            $pieces = $this->_writePortion($this->_xml, true);
            if (!$isOneFile) {
                $this->_renameFirstFile($pieces);
            }
        }

        if (!$isOneFile) {
            $this->_writeIndexFile($pieces);
        }
    }

    protected function _writePortion($chunk, $index = false)
    {
        $pieces = array();
        $path = $this->getPath();
        $io = new Varien_Io_File();
        $io->setAllowCreateFolders(true);
        $io->open(array('path' => $path));
        $name = $this->getSitemapFileName();
        $this->_iterator++;
        if (!$index) {
            $name = str_replace('.xml', '', $name);
            $name .= '_' . $this->_iterator . '.xml';
        }

        $fullPath = $path . $name;
        $pieces[] = $this->getDirUrl() . $name;
        $io->streamOpen($name);
        $io->streamWrite('<?xml version="1.0" encoding="UTF-8"?>' . PHP_EOL);
        $io->streamWrite('<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml" ');
        if ($this->getProductsThumbs() || $this->getCategoriesThumbs()) {
            $io->streamWrite('xmlns:image="http://www.google.com/schemas/sitemap-image/1.1"');
        }
        $io->streamWrite('>');

        $chinkCount = count($chunk);
        for ($i = 0; $i < $chinkCount; $i++) {
            if (false === strpos($chunk[$i], '<loc></loc>')) {
                $io->streamWrite($chunk[$i] . PHP_EOL);
                if (isset($chunk[$i + 1])) {
                    $fileSize = $this->getFileSizeWithNewLine($fullPath, $chunk[$i + 1]);
                    if ($this->getMaxFileSize() && $fileSize > ($this->getMaxFileSize() * 1024)) {
                        $newArray = array_slice($chunk, $i + 1);
                        $pieces = array_merge($pieces, $this->_writePortion($newArray, false));
                        break;
                    }
                }
            }
        }

        $io->streamWrite('</urlset>');
        $io->streamClose();
        if ($this->_isFirst) {
            $this->_firstTime = filemtime($path . $name);
            $this->_isFirst = false;
        }

        return $pieces;
    }

    protected function _generateCategories()
    {
        $result = array();
        if (!$this->getCategories()) {
            $this->finishSection();
            return $result;
        }

        /** @var Amasty_SeoToolKit_Model_Data $toolKitModel */
        $toolKitModel = Mage::getModel('amseotoolkit/data');
        $collection = $toolKitModel->getCategoryCollection();

        $pageSize = $this->getData('batch_size_cat')
            ? $this->getData('batch_size_cat') : self::DEFAULT_SIZE_CAT;
        $collection->setPageSize($pageSize);
        $pages = $collection->getLastPageNumber();
        $currentPage = $this->getStepCounter();

        if ($currentPage > $pages || $pages == 0) {
            //we should finish current section
            $this->addResponseMessage($this->getAmastyHelper()->__('Categories data was generated'));
            $this->finishSection();

            return $result;
        }

        $collection
            ->setCurPage($currentPage)
            ->load();

        $storeUrl = Mage::app()->getStore($this->_storeId)->getBaseUrl(Mage_Core_Model_Store::URL_TYPE_LINK);
        $changeFreq = $this->getCategoriesFrequency();
        $priority = $this->getCategoriesPriority();

        $hrefLangs = array();
        if ($this->getHreflangCat()) {
            $hrefLangs = Mage::helper('amseogooglesitemap/hrefurl')
                ->getCategoriesHreflangUrls($collection->getLoadedIds());
        }

        foreach ($collection as $item) {
            $item['url'] = $storeUrl . $item->getRequestPath();
            if (empty($item['url'])) {
                continue;
            }

            if (Mage::helper('amseogooglesitemap')->isEnterprise()) {
                $suffix = Mage::helper('catalog/category')->getCategoryUrlSuffix();
                if (!empty($suffix)) {
                    $item['url'] = $item['url'] . $suffix;
                }
            }

            if ($this->_isUrlToExclude($item['url'])) {
                continue;
            }

            $xmlLine = '<url><loc>%s</loc><priority>%.2f</priority><changefreq>%s</changefreq>';
            $xmlParams = array(
                htmlspecialchars($item['url']),
                $priority,
                $changeFreq
            );
            $xmlLine .= $this->writeHrefUrls($item, $hrefLangs);

            if ($this->getCategoriesThumbs() && $item->getImage()) {
                $xmlLine .= '<image:image>';
                if ($this->getCategoriesCaptions()) {
                    $xmlLine .= '<image:title>%s</image:title>';
                    $xmlParams[] = $item->getName();
                }
                $xmlLine .= '<image:loc>%s</image:loc></image:image>';
                $thumb = Mage::getBaseUrl('media') . 'catalog/category/' . $item->getImage();
                $xmlParams[] = htmlspecialchars($thumb);
            }

            if ($this->getCategoriesModified()) {
                $xmlLine .= '<lastmod>%s</lastmod>';
                $updateTime = strtotime($item->getUpdatedAt());
                $xmlParams[] = date('c', $updateTime);
            }

            $xmlLine .= '</url>';
            $result[] = vsprintf($xmlLine, $xmlParams);
        }

        $this->addPartAnswer($currentPage, $pageSize, count($result), 'Categories');
        $this->increaseCounter();

        unset($collection);

        return $result;
    }

    protected function _generateProducts()
    {
        $result = array();
        if (!$this->getProducts()) {
            $this->finishSection();

            return $result;
        }

        $changeFreq = $this->getProductsFrequency();
        $priority = $this->getProductsPriority();

        //set admin store for getting correct collection via using catalog Flat
        Mage::app()->setCurrentStore(0);

        /** @var Amasty_SeoToolKit_Model_Data $toolKitModel */
        $toolKitModel = Mage::getModel('amseotoolkit/data');
        $collection = $toolKitModel->getProductCollection($this->_storeId);
        if ($this->getData('prod_out_of_stock') == self::DISABLE_OUT_OF_STOCK) {
            Mage::getSingleton('cataloginventory/stock')->addInStockFilterToCollection($collection);
        }

        $pageSize = $this->getData('batch_size_prod') ?
            $this->getData('batch_size_prod') : self::DEFAULT_SIZE_PROD;
        $collection->setPageSize($pageSize);
        $pages = $collection->getLastPageNumber();
        $currentPage = $this->getStepCounter();

        if ($currentPage > $pages || $pages == 0) {
            //we should finish current section
            $this->addResponseMessage($this->getAmastyHelper()->__('Products data was generated'));
            $this->finishSection();

            return $result;
        }

        $needUseGetProductUrl = $this->getNeedUseProductUrl();
        $collection
            ->setCurPage($currentPage)
            ->load();

        Mage::app()->setCurrentStore($this->_storeId);
        $hrefLangs = array();
        if ($this->getHreflangProd()) {
            $hrefLangs = Mage::helper('amseogooglesitemap/hrefurl')
                ->getProductsHreflangUrls($collection->getLoadedIds());
        }

        foreach ($collection as $item) {
            $productUrl = Mage::getBaseUrl('web') . $item->getUrlPath();
            if ($needUseGetProductUrl) {
                $productUrl = $item->getProductUrl(false);
            }

            if ($this->_isUrlToExclude($productUrl)) {
                continue;
            }

            $xmlLine = '<url><loc>%s</loc><priority>%.2f</priority><changefreq>%s</changefreq>';
            $xmlParams = array(
                htmlspecialchars($productUrl),
                $priority,
                $changeFreq
            );
            $xmlLine .= $this->writeHrefUrls($item, $hrefLangs);

            if ($this->getProductsThumbs()) {
                $mediaGalleryImages = $item->load('media_gallery')->getMediaGalleryImages();
                $mainImageProcessed = false;
                $mainImage = $item->getImage();
                if ($mainImage && $mediaGalleryImages &&  $mediaGalleryImages->getSize()) {
                    foreach ($mediaGalleryImages as $galleryImage) {
                        $image = Mage::getModel('catalog/product_media_config')->getMediaUrl(
                            $galleryImage->getFile()
                        );
                        $isMainImage = $galleryImage->getFile() == $mainImage ? true : false;
                        if ($isMainImage) {
                            $mainImageProcessed = true;
                        }
                        $this->_getProductImageString(
                            $item,
                            $image,
                            $xmlLine,
                            $xmlParams,
                            $isMainImage,
                            $galleryImage->getLabel()
                        );
                    }
                }
                if (!$mainImageProcessed && $mainImage) {
                    $this->_getProductImageString(
                        $item,
                        Mage::getModel('catalog/product_media_config')->getMediaUrl($mainImage),
                        $xmlLine,
                        $xmlParams,
                        true
                    );
                }

            }

            if ($this->getProductsModified()) {
                $xmlLine .= '<lastmod>%s</lastmod>';
                $updateTime = strtotime($item->getUpdatedAt());
                $xmlParams[] = date('c', $updateTime);
            }
            $xmlLine .= '</url>';

            $result[] = vsprintf($xmlLine, $xmlParams);
        }
        $this->addPartAnswer($currentPage, $pageSize, count($result), 'Products');
        $this->increaseCounter();

        unset($collection);

        return $result;
    }

    /**
     * @param $item
     * @param array $hrefLangs
     * @return string
     */
    protected function writeHrefUrls($item, $hrefLangs, $relField = 'entity_id')
    {
        $result = '';
        if (isset($hrefLangs[$item->getData($relField)])) {
            foreach ($hrefLangs[$item->getData($relField)] as $lang => $url) {
                $result .= "<xhtml:link rel=\"alternate\" hreflang=\"$lang\" href=\"$url\"/>";
            }
        }

        return $result;
    }

    protected function _generateCms()
    {
        $result = array();
        if (!$this->getPages()) {
            $this->finishSection();

            return $result;
        }

        $changeFreq = $this->getPagesFrequency();
        $priority = $this->getPagesPriority();
        $collection = Mage::getModel('cms/page')->getCollection();
        $collection->addStoreFilter($this->_storeId);
        $collection->getSelect()->where('is_active = 1');

        $pageSize = $this->getData('batch_size_page')
            ? $this->getData('batch_size_page') : self::DEFAULT_SIZE_PAGE;

        $collection->setPageSize($pageSize);

        $pages = $collection->getLastPageNumber();
        $currentPage = $this->getStepCounter();
        if ($currentPage > $pages || $pages == 0) {
            //we should finish current section
            $this->addResponseMessage($this->getAmastyHelper()->__('Cms Pages data was generated'));
            $this->finishSection();

            return $result;
        }

        
        $collection
            ->setCurPage($currentPage)
            ->load();

        $hrefLangs = array();
        if ($this->getHreflangPage()) {
            $hrefLangs = Mage::helper('amseogooglesitemap/hrefurl')
                ->getCmsHreflangUrls($this->getRelationValues($collection));
        }
        foreach ($collection as $item) {
            $pageUrl = $this->_baseUrl . $item->getIdentifier();
            if (($this->getExcludeCmsAliases()
                    && in_array($item->getIdentifier(), $this->getExcludeCmsAliases())
                )
                || $this->_isUrlToExclude($pageUrl)
            ) {
                continue;
            }

            if ($item->getIdentifier() == Mage::getStoreConfig('web/default/cms_home_page', $this->_storeId)) {
                $pageUrl = Mage::app()->getStore($this->_storeId)->getBaseUrl();
            }

            $xmlLine = '<url><loc>%s</loc><priority>%.2f</priority><changefreq>%s</changefreq>';
            $xmlParams = array(
                htmlspecialchars($pageUrl),
                $priority,
                $changeFreq
            );
            $relField = Mage::helper('amseogooglesitemap/hrefurl')->getCmsRelationField();
            $xmlLine .= $this->writeHrefUrls($item, $hrefLangs, $relField);

            if ($this->getPagesModified()) {
                $xmlLine .= '<lastmod>%s</lastmod>';
                $updateTime = strtotime($item->getUpdateTime());
                $xmlParams[] = date('c', $updateTime);
            }
            $xmlLine .= '</url>';
            $result[] = vsprintf($xmlLine, $xmlParams);

        }
        $this->addPartAnswer($currentPage, $pageSize, count($result), 'Cms Pages');
        $this->increaseCounter();

        unset($collection);

        return $result;
    }

    protected function getRelationValues($collection)
    {
        $values = array();
        $relField = Mage::helper('amseogooglesitemap/hrefurl')->getCmsRelationField();
        foreach ($collection as $page) {
            $values[] = $page->getData($relField);
        }

        return array_unique(array_filter($values));
    }

    protected function getExcludeCmsAliases()
    {
        $exclude = $this->getData('exclude_cms_aliases');
        $exclude = str_replace(' ', '', $exclude);
        if ($exclude) {
            $exclude = explode("\r\n", $exclude);
            return $exclude;
        }

        return false;
    }

    protected function _generateExtra()
    {
        $result = array();
        if (!$this->getExtra()) {
            $this->finishSection();

            return $result;
        }

        $collection = explode(chr(13), $this->getExtraLinks());

        $changeFreq = $this->getExtraFrequency();
        $priority = $this->getExtraPriority();

        foreach ($collection as $item) {
            $result[] = sprintf(
                '<url><loc>%s</loc><changefreq>%s</changefreq><priority>%.2f</priority></url>',
                htmlspecialchars(trim($item)),
                $changeFreq,
                $priority
            );
        }

        unset($collection);

        $this->addResponseMessage($this->getAmastyHelper()->__('Extra links data was generated'));
        $this->finishSection();

        return $result;
    }

    protected function _generateTags()
    {
        $result = array();
        if (!$this->getTags()) {
            $this->finishSection();

            return $result;
        }

        $changeFreq = $this->getTagsFrequency();
        $priority = $this->getTagsPriority();

        $tags = Mage::getModel('tag/tag')->getResourceCollection()
            ->addPopularity()
            ->addStatusFilter(Mage::getModel('tag/tag')->getApprovedStatus())
            ->addStoreFilter($this->_storeId)
            ->setActiveFilter();

        $pageSize = $this->getData('batch_size_tag')
            ? $this->getData('batch_size_tag') : self::DEFAULT_SIZE_TAG;
        $tags->setPageSize($pageSize);
        $pages = $tags->getLastPageNumber();
        $currentPage = $this->getStepCounter();

        if ($currentPage > $pages || $pages == 0) {
            //we should finish current section
            $this->addResponseMessage($this->getAmastyHelper()->__('Tags data was generated'));
            $this->finishSection();

            return $result;
        }

        $tags->setCurPage($currentPage);

        foreach ($tags as $tag) {
            $tagUrl = $this->_baseUrl . 'tag/product/list/tagId/' . $tag->getTagId();
            if ($this->_isUrlToExclude($tagUrl)) {
                continue;
            }

            $result[] = sprintf(
                '<url><loc>%s</loc><changefreq>%s</changefreq><priority>%.2f</priority></url>',
                htmlspecialchars($tagUrl),
                $changeFreq,
                $priority
            );
        }
        $this->addPartAnswer($currentPage, $pageSize, count($result), 'Tags');
        $this->increaseCounter();

        unset($tags);

        return $result;
    }

    protected function _generateLanding()
    {
        $result = array();
        if (!Mage::helper('core')->isModuleEnabled('Amasty_Xlanding') || !$this->getLanding()) {
            $this->finishSection();
            return $result;
        }

        $changeFreq = $this->getLandingFrequency();
        $priority = $this->getLandingPriority();

        $landingPages = Mage::getModel('amlanding/resource_cms_page')->getCollection($this->_storeId);

        foreach ($landingPages as $page) {
            $title = $page->getTitle();
            if (isset($title)) {
                $landingUrl = $this->_baseUrl . $page->getUrl();
                if ($this->_isUrlToExclude($landingUrl)) {
                    continue;
                }

                $result[] = sprintf(
                    '<url><loc>%s</loc><changefreq>%s</changefreq><priority>%.2f</priority></url>',
                    htmlspecialchars($landingUrl),
                    $changeFreq,
                    $priority
                );
            }
        }


        $this->addPartAnswer(1, count($landingPages), count($result), 'Landing Pages');
        $this->addResponseMessage($this->getAmastyHelper()->__('Landing pages data was generated'));
        $this->finishSection();

        return $result;
    }

    protected function _generateNavigation()
    {
        $result = array();
        if (!Mage::helper('core')->isModuleEnabled('Amasty_Shopby')
            ||!$this->getNavigation()
        ) {
            $this->finishSection();

            return $result;
        }


        $changeFreq = $this->getNavigationFrequency();
        $priority = $this->getNavigationPriority();

        /** @var Amasty_Shopby_Model_Mysql4_Page_Collection $navigationPages */
        $navigationPages = Mage::getModel('amshopby/page')->getCollection();
        $navigationPages->addFieldToFilter('url', array('neq' => ''));
        $navigationPages->addStoreFilter();

        $pageSize = $this->getData('batch_size_page')
            ? $this->getData('batch_size_page') : self::DEFAULT_SIZE_PAGE;
        $navigationPages->setPageSize($pageSize);
        $pages = $navigationPages->getLastPageNumber();
        $currentPage = $this->getStepCounter();

        if ($currentPage > $pages || $pages == 0) {
            //we should finish current section
            $this->addResponseMessage($this->getAmastyHelper()->__('Navigation data was generated'));
            $this->finishSection();

            return $result;
        }

        $navigationPages->setCurPage($currentPage);

        foreach ($navigationPages as $navigationPage) {
            $url = $navigationPage->getUrl();
            if ($this->_isUrlToExclude($url)) {
                continue;
            }

            $result[] = sprintf(
                '<url><loc>%s</loc><changefreq>%s</changefreq><priority>%.2f</priority></url>',
                $url,
                $changeFreq,
                $priority
            );
        }


        unset($navigationPages);

        $this->addPartAnswer($currentPage, $pageSize, count($result), 'Navigation');
        $this->increaseCounter();

        return $result;
    }

    protected function _generateBlog()
    {
        $result = array();
        if (!Mage::helper('core')->isModuleEnabled('Magpleasure_Blog')
            && !$this->getBlog()
        ) {
            $this->finishSection();

            return $result;
        }

        $blogLinks = Mage::getModel('mpblog/sitemap')->generateLinks($this->_storeId);

        $changeFreq = $this->getBlogFrequency();
        $priority = $this->getBlogPriority();

        foreach ($blogLinks as $link) {
            if ($link['url']) {
                if ($this->_isUrlToExclude($link['url'])) {
                    continue;
                }

                $result[] = sprintf(
                    '<url><loc>%s</loc><changefreq>%s</changefreq><priority>%.2f</priority></url>',
                    str_replace('index.php/', '', $link['url']),
                    $changeFreq,
                    $priority
                );
            }
        }

        $this->addResponseMessage($this->getAmastyHelper()->__('Blog data was generated'));
        $this->finishSection(false);

        return $result;
    }


    protected function _generateBrands()
    {
        $result = array();
        if (!Mage::helper('core')->isModuleEnabled('Amasty_Shopby')
            || !$this->getBrands()
        ) {
            $this->finishSection();

            return $result;
        }

        $result = $this->_writeBrands(self::NAVIGATION);
        if (!$result) {
            $this->finishSection();
        } else {
            $this->addResponseMessage($this->getAmastyHelper()->__('Amasty_Shopby data was generated'));
            $this->finishSection(false);
        }

        return $result;
    }

    protected function _generateShopByBrands()
    {
        $result = array();
        if (!Mage::helper('core')->isModuleEnabled('Amasty_Brands')
            || !$this->getByBrands()
        ) {
            $this->finishSection();

            return $result;
        }

        $result = $this->_writeBrands(self::SHOP_BY_BRANDS);
        if (!$result) {
            $this->finishSection();
        } else {
            $this->addResponseMessage($this->getAmastyHelper()->__('Amasty_Brands data was generated'));
            $this->finishSection(false);
        }

        return $result;
    }

    protected function _writeBrands($type)
    {
        $result = array();

        if ($type == self::SHOP_BY_BRANDS) {
            $changeFreq = $this->getByBrandsFrequency();
            $priority = $this->getByBrandsPriority();
            $attrCode = trim(Mage::getStoreConfig('ambrands/general/attribute'));
            /** @var Amasty_Brands_Helper_Data $helper */
            $helper = Mage::helper('ambrands');
        } elseif ($type == self::NAVIGATION) {
            $changeFreq = $this->getBrandsFrequency();
            $priority = $this->getBrandsPriority();
            $attrCode = trim(Mage::getStoreConfig('amshopby/brands/attr'));
            /** @var Amasty_Shopby_Helper_Url $helper */
            $helper = Mage::helper('amshopby/url');
        } else {
            return $result;
        }

        if (!$attrCode) {
            return $result;
        }

        $brandPages = Mage::getModel('eav/config')->getAttribute(
            Mage_Catalog_Model_Product::ENTITY,
            $attrCode
        );

        foreach ($brandPages->getSource()->getAllOptions() as $option) {
            if ($option['value']) {
                $entityTypeId = Mage::getModel('eav/entity')->setType('catalog_product')->getTypeId();
                /** @var Mage_Eav_Model_Attribute $attribute */
                $attribute = Mage::getModel('catalog/resource_eav_attribute')
                    ->loadByCode($entityTypeId, $attrCode);

                if (!$attribute->getId()) {
                    continue;
                }

                $options = $attribute->getFrontend()->getSelectOptions();
                array_shift($options);

                $filter = new Varien_Object();
                // important when used at category pages
                $layer = Mage::getModel('catalog/layer')
                    ->setCurrentCategory(Mage::app()->getStore()->getRootCategoryId());

                $filter->setLayer($layer);
                $filter->setStoreId(Mage::app()->getStore()->getId());
                $filter->setAttributeModel($attribute);

                if ($type == self::NAVIGATION) {
                    $optionsCount = Mage::getResourceModel('catalog/layer_filter_attribute')->getCount($filter);
                    if (!array_key_exists($option['value'], $optionsCount)) {
                        continue;
                    }
                }

                $url = $helper->getOptionUrl($attrCode, $option['value']);
                if ($this->_isUrlToExclude($url) || !$url) {
                    continue;
                }

                $result[] = sprintf(
                    '<url><loc>%s</loc><changefreq>%s</changefreq><priority>%.2f</priority></url>',
                    $url,
                    $changeFreq,
                    $priority
                );
            }
        }


        unset($brandPages);

        return $result;
    }

    public function getExternalLinks($storeId)
    {
        $profiles = $this->getResourceCollection()->addFieldToFilter('stores', $storeId);

        $links = array();
        foreach ($profiles as $profile) {
            $collection = explode(chr(13), $profile->getExtraLinks());
            if (count($collection) > 0) {
                foreach ($collection as $link) {
                    $link = trim($link);
                    $links[$link] = $link;
                }
            }
        }

        return $links;
    }

    protected function _writeIndexFile($pieces)
    {
        $io = new Varien_Io_File();
        $io->setAllowCreateFolders(true);
        $io->open(array('path' => $this->getPath()));

        $this->_date = Mage::getSingleton('core/date')->gmtDate('Y-m-d');
        $this->_baseUrl = Mage::app()->getStore($this->_storeId)->getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB);

        $name = $this->getSitemapFileName();
        $io->streamOpen($name);
        $io->streamWrite('<?xml version="1.0" encoding="UTF-8"?>' . "\n");
        $io->streamWrite('<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">');
        foreach ($pieces as $url) {
            $item = sprintf(
                '<sitemap><loc>%s</loc><lastmod>%s</lastmod></sitemap>',
                htmlspecialchars($this->_baseUrl . $url),
                $this->_date
            );
            $io->streamWrite($item . "\n");
        }
        $io->streamWrite('</sitemapindex>');
        $io->streamClose();
    }

    protected function _renameFirstFile(&$chunks)
    {
        $path = $this->getPath();
        $io = new Varien_Io_File();
        $io->setAllowCreateFolders(true);
        $io->open(array('path' => $path));

        $name = $this->getSitemapFileName();
        $newFileName = str_replace('.xml', '', $name);
        $newFileName .= '_1.xml';

        $chunks[0] = $this->getDirUrl() . $newFileName;

        $io->cp($name, $newFileName);
        unlink($path . $name);
    }


    /**
     * @param Mage_Catalog_Model_Product $item
     * @param string $image
     * @param string $xmlLine
     * @param array $xmlParams
     * @param bool $mainImage
     */
    protected function _getProductImageString($item, $image, &$xmlLine, &$xmlParams, $mainImage = false, $label = '')
    {
        if ($image && ($mainImage || $this->getProductsCaptionsAll())) {
            $xmlLine .= '<image:image>';

            if ($this->getProductsCaptions()) {
                if (!$label && $mainImage) {
                    $label = $item->getThumbnailLabel();
                }
                if ($label == '') {
                    $label = $this->parsePlaceholder($item);
                }

                $label = htmlspecialchars($label);
                $xmlLine .= '<image:title>%s</image:title>';
                $xmlLine .= '<image:caption>%s</image:caption>';
                $label = ($label == '') ? ' ' : $label;
                $xmlParams[] = $label; //for title
                $xmlParams[] = $label; //for caption
            }

            $xmlLine .= '<image:loc>%s</image:loc></image:image>';
            $xmlParams[] = htmlspecialchars($image);
        }
    }
}
