<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2018 Amasty (https://www.amasty.com)
 * @package Amasty_SeoGoogleSitemap
 */


class Amasty_SeoGoogleSitemap_Model_SitemapAbstract extends Mage_Core_Model_Abstract
{
    CONST COUNTER = 'amseogooglesitemap_counter';
    CONST CURRENT_STEP = 'amseogooglesitemap_current_step';
    CONST CURRENT_SECTION = 'amseogooglesitemap_current_section';
    CONST GENERATION_DONE = 'amseogooglesitemap_generation_done';
    CONST SESSION_MESSAGES = 'amseogooglesitemap_messages';

    protected $_date;
    protected $_xml = array();
    protected $_baseUrl;
    protected $_storeId;
    protected $_iterator = 1;
    protected $_excludeUrls;
    protected $_isFirst = true;
    protected $_firstTime;

    /**
     * Generation step order
     */
    protected $_generationSteps = array(
        0 => 'Categories',
        1 => 'Products',
        2 => 'Cms',
        3 => 'Extra',
        4 => 'Tags',
        5 => 'Landing',
        6 => 'Brands',
        7 => 'Navigation',
        8 => 'ShopByBrands',
        9 => 'Blog',
        10 => 'Empty_for_showing_last_results'
    );

    public function _construct()
    {
        parent::_construct();
        $this->_init('amseogooglesitemap/sitemap');
    }

    protected function increaseCounter()
    {
        $counter = $this->getSession()->getData(self::COUNTER);
        $this->getSession()->setData(self::COUNTER, ++$counter);
    }

    protected function getSession()
    {
        return Mage::getSingleton('core/session');
    }

    /**
     * @return Amasty_SeoGoogleSitemap_Helper_Data
     */
    protected function getAmastyHelper()
    {
        return Mage::helper('amseogooglesitemap');
    }

    protected function getCurrentStep()
    {
        return $this->getSession()->getData(self::CURRENT_STEP);
    }

    public function clearSession()
    {
        $this->getSession()->setData(self::CURRENT_STEP, 1);
        $this->getSession()->setData(self::SESSION_MESSAGES, array());
        $this->getSession()->setData(self::CURRENT_SECTION, 0);
        $this->getSession()->setData(self::GENERATION_DONE, 0);
        $this->getSession()->setData(self::COUNTER, 1);
        $this->getSession()->setData(Amasty_SeoGoogleSitemap_Helper_Hrefurl::CURRENT_HREFLANG_CODES, null);
    }

    protected function getStepCounter()
    {
        return $this->getSession()->getData(self::COUNTER);
    }

    protected function isAllGenerationDone()
    {
        return (bool)$this->getSession()->getData(self::GENERATION_DONE);
    }

    protected function initializeGeneration()
    {
        $this->_storeId = $this->getStores();
        $this->_date = Mage::getSingleton('core/date')->gmtDate('Y-m-d');
        $this->_baseUrl = Mage::app()->getStore($this->_storeId)->getBaseUrl(Mage_Core_Model_Store::URL_TYPE_LINK);
    }

    protected function flushSeoSingleUrlCache()
    {
        if (Mage::helper('core')->isModuleEnabled('Amasty_SeoSingleUrl')) {
            /** @var Amasty_SeoSingleUrl_Helper_Product_Url_Rewrite $rewriteHelper */
            $rewriteHelper = Mage::helper('amseourl/product_url_rewrite');
            $rewriteHelper->flushProductUrlCache();
        }
    }

    protected function finishSection($withGeneration = true)
    {
        $this->increaseSection();
        $this->resetCounter();

        if ($withGeneration) {
            $this->generateSiteStepData();
        }
    }

    protected function addResponseMessage($message)
    {
        $messages = $this->getSession()->getData(self::SESSION_MESSAGES);
        if (!$messages) {
            $messages = array();
        }

        $messages[] = $message;
        $this->getSession()->setData(self::SESSION_MESSAGES, $messages);
    }



    public function getSitemapFileName()
    {
        $filename = pathinfo($this->getFolderName());

        return $filename['basename'];
    }

    /**
     * Return full file name with path
     *
     * @return string
     */
    public function getPreparedFilename()
    {
        return $this->getPath() . $this->getSitemapFilename();
    }

    protected function increaseSection()
    {
        $currentSection = $this->getCurrentSection();
        $this->getSession()->setData(self::CURRENT_SECTION, ++$currentSection);
    }

    protected function resetCounter()
    {
        $this->getSession()->setData(self::COUNTER, 1);
    }

    protected function getCurrentSection()
    {
        return $this->getSession()->getData(self::CURRENT_SECTION);
    }

    public function parsePlaceholder($product)
    {
        $txt = $this->getProductsCaptionsTemplate();
        if ($txt == '') {
            return $txt;
        }

        $vars = array();
        preg_match_all('/{([a-zA-Z:\_0-9]+)}/', $txt, $vars);
        if (!$vars[1]) {
            return $txt;
        }
        $vars = $vars[1];

        foreach ($vars as $var) {
            $value = '';
            switch ($var) {
                case 'product_name':
                    $value = $product->getName();
                    break;
            }
            $txt = str_replace('{' . $var . '}', $value, $txt);
        }

        return $txt;
    }

    protected function _beforeSave()
    {
        $io = new Varien_Io_File();
        $realPath = $io->getCleanPath($this->getPath());

        /**
         * Check path is allow
         */
        if (!$io->allowedPath($realPath, Mage::getBaseDir())) {
            Mage::throwException(Mage::helper('sitemap')->__('Please define correct path'));
        }
        /**
         * Check exists and writeable path
         */
        if (!$io->fileExists($realPath, false)) {
            Mage::throwException(
                Mage::helper(
                    'sitemap'
                )->__(
                    'Please create the specified folder "%s" before saving the sitemap.',
                    Mage::helper('core')->htmlEscape($this->getPreparedFilename())
                )
            );
        }

        if (!$io->isWriteable($realPath)) {
            Mage::throwException(
                Mage::helper('sitemap')->__(
                    'Please make sure that "%s" is writable by web-server.',
                    $this->getPreparedFilename()
                )
            );
        }

        /**
         * Check allow filename
         */
        if (!preg_match('#\.xml$#', $this->getSitemapFilename())) {
            $this->setSitemapFilename($this->getSitemapFilename() . '.xml');
        }

        $search = str_replace('\\', '/', Mage::getBaseDir());
        $siteMapPath = str_replace($search, '', $realPath);
        $siteMapPath = rtrim($siteMapPath, '/');
        $this->setSitemapPath($siteMapPath . '/');

        return parent::_beforeSave();
    }

    protected function removeTmpFiles()
    {
        $io = new Varien_Io_File();
        $realPath = $io->getCleanPath($this->getPath());
        $name = $this->getSitemapFileName();
        $tmpName = str_replace('.xml', '_tmp.xml', $name);

        if ($io->fileExists($realPath . $tmpName)) {
            unlink($realPath . $tmpName);
        }
    }

    protected function _removeOldFiles()
    {
        $io = new Varien_Io_File();
        $realPath = $io->getCleanPath($this->getPath());
        $fileName = $this->getSitemapFilename();
        $pos = strpos($fileName, ".xml");
        $noExtensionFileName = substr($fileName, 0, $pos);
        $fullFilePath = $realPath . $noExtensionFileName . "_*";
        $files = glob($fullFilePath);

        foreach ($files as $file) {
            if (filemtime($file) < $this->_firstTime) {
                unlink($file);
            }
        }
    }

    /**
     * Return real file path
     *
     * @return string
     */
    protected function getPath()
    {
        if (is_null($this->_filePath)) {
            $dirname = pathinfo($this->getFolderName());
            if ($dirname['dirname'] == '.') {
                $this->_filePath = str_replace('//', '/', Mage::getBaseDir() . '/');
            } else {
                $this->_filePath = str_replace('//', '/', Mage::getBaseDir() . '/' . $dirname['dirname'] . '/');
            }
        }

        return $this->_filePath;
    }

    protected function getDirUrl()
    {
        $dirname = pathinfo($this->getFolderName());
        $dirPath = $dirname['dirname'] . '/';

        if ($dirname['dirname'] == '.') {
            $dirPath = '';
        }

        return $dirPath;
    }

    protected function getFileSizeWithNewLine($testFile, $line)
    {
        $size = filesize($testFile) + strlen($line) + strlen('</urlset>');

        return $size;
    }

    protected function getNeedUseProductUrl()
    {
        $needUseGetProductUrl = false;

        $magentoVersion = Mage::getVersion();
        if (version_compare($magentoVersion, '1.8', '>=')) {
            $needUseGetProductUrl = true;
        }

        $isUniqueProductUrl = Mage::helper('core')->isModuleEnabled('Amasty_SeoSingleUrl');
        if ($isUniqueProductUrl && Mage::getStoreConfig('amseourl/general/product_url_type') != 0) {
            $needUseGetProductUrl = true;
        }

        return $needUseGetProductUrl;
    }

    protected function _isUrlToExclude($url)
    {
        $isToExclude = false;

        if (empty($this->_excludeUrls)) {
            $this->_excludeUrls = $this->getExcludeUrls();
        }

        if ($this->_excludeUrls != '' && strpos($this->_excludeUrls, $url) !== false) {
            $isToExclude = true;
        }

        return $isToExclude;
    }

    protected function generateMethodName($step)
    {
        return '_generate' . $step;
    }

    protected function addPartAnswer($currentPage, $pageSize, $resultCount, $sectionName)
    {
        $start = ($currentPage - 1) * $pageSize + 1;
        if ($resultCount == 0) {
            $resultCount = 1;
        }
        $this->addResponseMessage(
            $this->getAmastyHelper()->__(
                $sectionName . ' data was generated for ids: %s to %s.',
                $start,
                $start + $resultCount - 1
            )
        );
    }

    public function clearLogs()
    {
        Mage::getSingleton('core/session')->setData(
            Amasty_SeoGoogleSitemap_Model_Sitemap::SESSION_MESSAGES,
            array()
        );
    }

    public function getLogs()
    {
        return Mage::getSingleton('core/session')->getData(
            Amasty_SeoGoogleSitemap_Model_Sitemap::SESSION_MESSAGES
        );
    }
}
