<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2018 Amasty (https://www.amasty.com)
 * @package Amasty_SeoGoogleSitemap
 */


class Amasty_SeoGoogleSitemap_Adminhtml_Amseogooglesitemap_SitemapController
    extends Mage_Adminhtml_Controller_Action
{
    public function indexAction()
    {
        $this->_title($this->getAmastyHelper()->__('Manage Sitemap Pages'));

        $this->loadLayout();
        $this->_addContent($this->getLayout()->createBlock('amseogooglesitemap/adminhtml_sitemap'));
        $this->renderLayout();
    }

    public function editAction()
    {
        $id = (int)$this->getRequest()->getParam('id');
        $model = Mage::getModel('amseogooglesitemap/sitemap')->load($id);

        if ($model->getId() || $id == 0) {
            $data = Mage::getSingleton('adminhtml/session')->getFormData(true);

            if (!empty($data)) {
                $model->setData($data);
            }

            Mage::register('am_sitemap_profile', $model);

            $this->loadLayout();
            $this->_setActiveMenu('amseogooglesitemap/items');

            $this->_addBreadcrumb(
                Mage::helper('adminhtml')->__('Item Manager'),
                Mage::helper('adminhtml')->__('Item Manager')
            );

            $this->_addBreadcrumb(
                Mage::helper('adminhtml')->__('Item News'),
                Mage::helper('adminhtml')->__('Item News')
            );

            $this->_addContent($this->getLayout()->createBlock('amseogooglesitemap/adminhtml_sitemap_edit'))
                ->_addLeft($this->getLayout()->createBlock('amseogooglesitemap/adminhtml_sitemap_edit_tabs'));

            $this->renderLayout();
        } else {
            Mage::getSingleton('adminhtml/session')->addError(
                $this->getAmastyHelper()->__('Item does not exist')
            );
            $this->_redirect('*/*/');
        }
    }



    public function updateurlAction()
    {
        $html = array();

        $id = (int)$this->getRequest()->getParam('id');
        $model = Mage::getModel('amseogooglesitemap/sitemap')->load($id);
        if ($model->getId()) {
            $html['id'] = $id;
            $html['html'] = Mage::helper('amseogooglesitemap')->getFileLinkHtml($model->getFolderName());
        }

        $html = Zend_Json::encode($html);
        $this->getResponse()->setBody($html);
    }

    public function generateAction()
    {
        $result = array(
            'continue' => 0,
            'log' => array(),
            'error' => ''
        );

        $id = (int)$this->getRequest()->getParam('id');
        $clear = (int)$this->getRequest()->getParam('clear');
        try {
            /* @var $model Amasty_SeoGoogleSitemap_Model_Sitemap */
            $model = Mage::getModel('amseogooglesitemap/sitemap')->load($id);
            if (!$model->getId()) {
                throw new Exception($this->getAmastyHelper()->__('Generation Item does not exist'));
            }

            if ($clear) {
                $model->clearSession();
            }

            $model->run();

            $logs = $model->getLogs();

            if ($logs) {
                $result['log'] = $logs;
                $model->clearLogs();
                $result['continue'] = 1;
            } else {
                Mage::throwException($this->getAmastyHelper()->__('Generation was stopped. Empty result.'));
            }

        } catch (\Exception $exception) {
            $result['error'] = $exception->getMessage();
            $result['log'] = array();
            $result['continue'] = 0;
        }

        $result = Zend_Json::encode($result);
        $this->getResponse()->setBody($result);
    }

    /**
     * @return Amasty_SeoGoogleSitemap_Helper_Data
     */
    protected function getAmastyHelper()
    {
        return Mage::helper('amseogooglesitemap');
    }

    public function newAction()
    {
        $this->_forward('edit');
    }

    public function saveAction()
    {
        $profileId = (int)$this->getRequest()->getParam('id');
        if ($data = $this->getRequest()->getPost()) {
            $profile = Mage::getModel('amseogooglesitemap/sitemap');

            try {
                $this->validateData($data);
                $profile->setData($data)->setId($this->getRequest()->getParam('id'));
                $profile->save();
                $profileId = $profile->getId();

                Mage::getSingleton('adminhtml/session')->addSuccess(
                    $this->getAmastyHelper()->__('Profile was successfully saved')
                );
                Mage::getSingleton('adminhtml/session')->setFormData(false);

                if ($this->getRequest()->getParam('back')) {
                    $this->_redirect('*/*/edit', array('id' => $profileId));

                    return;
                }
                $this->_redirect('*/*/');

                return;
            } catch (Exception $e) {
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
                Mage::getSingleton('adminhtml/session')->setFormData($data);
                $params = array();
                if ($profileId) {
                    $params['id'] = $profileId;
                }
                $this->_redirect('*/*/edit', $params);

                return;
            }
        }

        Mage::getSingleton('adminhtml/session')->addError(
            $this->getAmastyHelper()->__('Unable to find item to save')
        );
        $this->_redirect('*/*/');
    }

    protected function validateData($data)
    {
        if (!isset($data['folder_name'])) {
            Mage::throwException(
                $this->getAmastyHelper()->__('Please specify Path to sitemap file')
            );
        }

        if (strpos($data['folder_name'], 'app/') !== false) {
            Mage::throwException(
                $this->getAmastyHelper()->__('You can\'t write into \'app\' folder.')
            );
        }

        $extensionArray = explode('.', $data['folder_name']);
        $extension = end($extensionArray);
        if ($extension !== 'xml') {
            Mage::throwException(
                $this->getAmastyHelper()->__('Please specify Path to sitemap file with XML extension')
            );
        }
    }

    public function deleteAction()
    {
        if ($this->getRequest()->getParam('id') > 0) {
            try {
                $model = Mage::getModel('amseogooglesitemap/sitemap');

                $model->setId($this->getRequest()->getParam('id'))
                    ->delete();

                Mage::getSingleton('adminhtml/session')->addSuccess(
                    Mage::helper('adminhtml')->__('Item was successfully deleted')
                );
                $this->_redirect('*/*/');
            } catch (Exception $e) {
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
                $this->_redirect('*/*/edit', array('id' => $this->getRequest()->getParam('id')));
            }
        }
        $this->_redirect('*/*/');
    }

    public function massDeleteAction()
    {
        $sitemapsIds = $this->getRequest()->getParam('sitemaps');
        if (! is_array($sitemapsIds)) {
            Mage::getSingleton('adminhtml/session')->addError(Mage::helper('adminhtml')->__('Please select item(s)'));
        } else {
            try {
                foreach ($sitemapsIds as $sitemapId) {
                    $sitemaps = Mage::getModel('amseogooglesitemap/sitemap')->load($sitemapId);
                    $sitemaps->delete();
                }
                Mage::getSingleton('adminhtml/session')->addSuccess(
                    Mage::helper('adminhtml')->__(
                        'Total of %d record(s) were successfully deleted', count($sitemapsIds)
                    )
                );
            } catch (Exception $e) {
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
            }
        }
        $this->_redirect('*/*/index');
    }

    public function massDuplicateAction()
    {
        $sitemapId = $this->getRequest()->getParam('id');

        try {
            $sitemaps = Mage::getModel('amseogooglesitemap/sitemap')->load($sitemapId);
            $data     = $sitemaps->getData();
            unset($data['id']);
            Mage::getModel('amseogooglesitemap/sitemap')->setData($data)->save();

            Mage::getSingleton('adminhtml/session')->addSuccess(
                Mage::helper('adminhtml')->__(
                    'Sitemap was successfully duplicated'
                )
            );
        } catch (Exception $e) {
            Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
        }

        $this->_redirect('*/*/index');
    }

    public function exportCsvAction()
    {
        $this->_exportGrid('csv');
    }

    public function exportXmlAction()
    {
        $this->_exportGrid('xml');
    }

    /**
     * @param $type
     *
     * @throws Exception
     */
    protected function _exportGrid($type)
    {
        $fileName = 'amasty_sitemap_admin_export.' . $type;
        $block    = $this->getLayout()->createBlock('amseogooglesitemap/adminhtml_sitemap_grid');
        switch ($type) {
            case 'xml' :
                $content = $block->getXml();
                break;

            case 'csv' :
                $content = $block->getCsv();
                break;

            default :
                throw new Exception('Please specify export data type');
                break;
        }

        $this->_prepareDownloadResponse($fileName, $content);
    }

    public function renderLayout($output = '')
    {
        $this->_setActiveMenu('cms/amseotoolkit/amseogooglesitemap');
        parent::renderLayout();
    }

    protected function _isAllowed()
    {
        return Mage::getSingleton('admin/session')->isAllowed('cms/amseotoolkit/amseogooglesitemap');
    }
}
