﻿<?php

class Dv_Labels_Model_Xml extends Mage_Core_Model_Abstract
{

    // If $xmlTypeId = null then generation is auto
    public function generateXml($xmlTypeId = null, $weight = null, $grandTotal = null )
    {
        if (is_null($xmlTypeId)){
            $xmlTypeId = $this->getXmlTypeId();
        }

        $order = $this->getOrder();
        $storeId = $order->getStoreId();
        if (is_null($grandTotal)){
            $grandTotal = $order->getGrandTotal();
        }

        if (is_null($weight)){
            $weight = $order->getWeight();
			if($weight < "0.150") {
				$weight = "0.151";
			}
        }

        if ( $xmlTypeId == Dv_Labels_Model_Source_Xmltypes::XML_TYPE_REGISTERED_MAIL_ID && $order->getShippingAddress()->getCountry() !== 'SE'){
            // NOTE: If Customer Country is not SE then BREKD should be changed to BREKI
            $xmlTypeId = Dv_Labels_Model_Source_Xmltypes::XML_TYPE_REGISTERED_MAIL_ID_NOT_SE;
        } elseif ( $xmlTypeId == Dv_Labels_Model_Source_Xmltypes::XML_TYPE_CACHE_ON_DELIVERY_ID && $weight > $this->_getConfig('xmllabels/generation/weight_cod', $storeId) ){
            // NOTE: If $weight > 2(by default) then PAF should be changed to P25
            $xmlTypeId = Dv_Labels_Model_Source_Xmltypes::XML_TYPE_CACHE_ON_DELIVERY_ID_GT2;
        }

        $customPath = $this->_getConfig('xmllabels/general/local_xml_folder', $storeId);
        $path = Mage::getBaseDir() . DS . $customPath;
        if (!file_exists($path)) { mkdir($path); }
        $name = md5(microtime());
        $file = $path . DS . $name . '.xml';

        $doc = new DOMDocument('1.0', 'iso-8859-1');
        $doc->formatOutput = true;
        $data = $doc->createElement('data');
        $doc->appendChild($data);

        $dataArray = $this->_getAttributesForXml();

        //SENDER section
        $sender = $doc->createElement('sender');
        $senderAttr = $doc->createAttribute('sndid');
        $senderAttr->value = '1';
        $sender->appendChild($senderAttr);
        $data->appendChild($sender);
        foreach ($dataArray['sender'] as $itemName){
            $element = $doc->createElement('val', $this->_getConfig('xmllabels/sender/' . $itemName, $storeId));
            $attribute = $doc->createAttribute('n');
            $attribute->value = $itemName;
            $element->appendChild($attribute);
            $sender->appendChild($element);
        }

        //RECEIVER section
        $receiver = $doc->createElement('receiver');
        $receiverAttr = $doc->createAttribute('rcvid');
        $receiverAttr->value = $order->getCustomerId();
        $receiver->appendChild($receiverAttr);
        $data->appendChild($receiver);
        foreach ($dataArray['receiver'] as $itemName => $itemValue){
            $element = $doc->createElement('val', $itemValue);
            $attribute = $doc->createAttribute('n');
            $attribute->value = $itemName;
            $element->appendChild($attribute);
            $receiver->appendChild($element);
        }

        //SHIPMENT section
        $shipment = $doc->createElement('shipment');
        $shipmentAttr = $doc->createAttribute('orderno');
        $shipmentAttr->value = $order->getRealOrderId();
        $shipment->appendChild($shipmentAttr);
        $data->appendChild($shipment);

        $from = $doc->createElement('val', $this->_getConfig('xmllabels/sender/sin', $storeId));
        $fromAttr = $doc->createAttribute('n');
        $fromAttr->value = 'from';
        $from->appendChild($fromAttr);
        $shipment->appendChild($from);

        $to = $doc->createElement('val', $order->getCustomerId());
        $toAttr = $doc->createAttribute('n');
        $toAttr->value = 'to';
        $to->appendChild($toAttr);
        $shipment->appendChild($to);

        //SERVICE
        $service = $doc->createElement('service');
        $serviceAttr = $doc->createAttribute('srvid');
        $serviceAttr->value = $xmlTypeId;
        $service->appendChild($serviceAttr);
        $shipment->appendChild($service);

	    // If we've chosen to print as Postförskott (Cash On Delivery) from the order view page the below addon included. Otherwise it is not. -->
        if ($xmlTypeId == Dv_Labels_Model_Source_Xmltypes::XML_TYPE_CACHE_ON_DELIVERY_ID || $xmlTypeId == Dv_Labels_Model_Source_Xmltypes::XML_TYPE_CACHE_ON_DELIVERY_ID_GT2){
            $addon = $doc->createElement('addon');
            $addonAttr = $doc->createAttribute('adnid');
            $addonAttr->value = 'COD';
            $addon->appendChild($addonAttr);
            $service->appendChild($addon);

            //Grand Total including tax
            $amount = $doc->createElement('val', $grandTotal);
            $amountAttr = $doc->createAttribute('n');
            $amountAttr->value = 'amount';
            $amount->appendChild($amountAttr);
            $addon->appendChild($amount);
        }




        // Customer Text/Letter Message notification
        $addon = $doc->createElement('addon');
        $addonAttr = $doc->createAttribute('adnid');
        // If customer phone number is a swedish cell phone number they are to be notified by text message, otherwise they are to be notified by letter.
        if ( $this->_isCell($order->getShippingAddress()->getTelephone(), $order->getShippingAddress()->getCountry()) ){
            ///Text Message
            $addonAttr->value = 'NOTSMS';
        } else {
            //Letter notification
            $addonAttr->value = 'NOTLTR';
        }
        $addon->appendChild($addonAttr);

        $service->appendChild($addon);
        //SERVICE end


        //RETURN label
/*
        if($storeId == '14') {
            $returnLabel = $doc->createElement('val', 'both');
            $returnLabelAttr = $doc->createAttribute('n');
            $returnLabelAttr->value = 'returnlabel';
            $returnLabel->appendChild($returnLabelAttr);
            $service->appendChild($returnLabel);
        }
*/
        //RETURN end

        //CONTAINER
        $container = $doc->createElement('container');
        $containerAttr = $doc->createAttribute('type');
        $containerAttr->value = 'parcel';
        $container->appendChild($containerAttr);
        $shipment->appendChild($container);

        $contents = $doc->createElement('val', $order->getRealOrderId());
        $contentsAttr = $doc->createAttribute('n');
        $contentsAttr->value = 'contents';
        $contents->appendChild($contentsAttr);
        $container->appendChild($contents);

        $contents = $doc->createElement('val', $weight);
        $contentsAttr = $doc->createAttribute('n');
        $contentsAttr->value = 'weight';
        $contents->appendChild($contentsAttr);
        $container->appendChild($contents);

        $doc->save($file);
        $doc->saveXML();
        return $file;
    }

    protected function _getAttributesForXml()
    {
        $order = $this->getOrder();
        $shippingAddress = $order->getShippingAddress();
        $streetLines = $shippingAddress->getStreet();

		/* Add + to country numbers */
		if (strpos($telephone,'45') !== false && strpos($telephone,'+') !== true) {
			$telephone = "00" . $telephone;
		}
		$telephone = $shippingAddress->getTelephone();
		if (strpos($telephone,'46') !== false && strpos($telephone,'+') !== true) {
			$telephone = "+" . $telephone;
		}
		if (strpos($telephone,'47') !== false && strpos($telephone,'+') !== true) {
			$telephone = "+" . $telephone;
		}
		if (strpos($telephone,'358') !== false && strpos($telephone,'+') !== true) {
			$telephone = "+" . $telephone;
		}

        $country = $shippingAddress->getCountry();

        $data = array(
            'sender'    => array('name', 'address1', 'address2', 'zipcode', 'city', 'country', 'contact', 'phone', 'email', 'orgno'),
            'receiver'  => array(
//                'name'      =>  $order->getCustomerName(),
                'name'      => $shippingAddress->getFirstname() . " " . $shippingAddress->getLastname(),
                'address1'  => isset($streetLines[0]) ? $streetLines[0] : '',
                'address2'  => isset($streetLines[1]) ? $streetLines[1] : '',
                'zipcode'   => $shippingAddress->getPostcode(),
                'city'      => $shippingAddress->getCity(),
                'country'   => $shippingAddress->getCountry(),
                'contact'   => $order->getCustomerName(),
                'phone'     => $telephone,
                'fax'       => $shippingAddress->getFax(),
                'email'     => $order->getCustomerEmail()
            )
        );

        $_cellPhone = $this->_isCell($telephone) ? $telephone : '';
        if ($_cellPhone){
            $data['receiver']['sms'] = $_cellPhone;
        }
        return $data;
    }

    public function getOrder()
    {
        if (!$this->getData('order')){
            if ( $order = Mage::registry('sales_order') ){
                $this->setData('order', $order);
            } else if ($order = Mage::registry('current_order')){
                $this->setData('order', $order);
            } else {
                Mage::throwException($this->__('The order is not specified.'));
            }
        }
        return $this->getData('order');
    }

    public function getXmlTypeId()
    {
        $order = $this->getOrder();
        $storeId = $order->getStoreId();
        $state = $order->getState();
        if ($order->isCanceled() || $state === Mage_Sales_Model_Order::STATE_CLOSED) {
            Mage::throwException($this->__('The order is CLOSED.'));
        }

        $this->createInvoice();
        $this->createShipment();
        if ($this->getIsOrderChanged()){
            $order->setData('updated_at', date("Y-m-d H:i:s") )
                ->setIsCronUpdate(true)
                ->save();
        }

        $_weight = $order->getWeight();
        $_limitWeight = $this->_getConfig('xmllabels/generation/weight', $storeId);
        $_grandTotal = $order->getGrandTotal();
        $_limitCost = $this->_getConfig('xmllabels/generation/cost', $storeId);

        if ($_weight > $_limitWeight && $order->getShippingAddress()->getCountry() !== 'SE' && $order->getShippingAddress()->getCountry() !== 'DK' && $order->getShippingAddress()->getCountry() !== 'NO' && $order->getShippingAddress()->getCountry() !== 'FI') {
			$xmlTypeId = Dv_Labels_Model_Source_Xmltypes::XML_TYPE_PACKAGE_INT_ID;
		} elseif ($_weight > $_limitWeight) {
            $xmlTypeId = Dv_Labels_Model_Source_Xmltypes::XML_TYPE_DHL_SERVICE_POINT_ID;
        } elseif ($_grandTotal > $_limitCost){
            $xmlTypeId = Dv_Labels_Model_Source_Xmltypes::XML_TYPE_REGISTERED_MAIL_ID;
        } else {
            $xmlTypeId = Dv_Labels_Model_Source_Xmltypes::XML_TYPE_VARUBREV_ID;
        }

        return $xmlTypeId;
    }

    public function createInvoice()
    {
        $order = $this->getOrder();
        try {
            if ($invoice = $this->_initInvoice($order)) {
                $invoice->register();
                $invoice->setEmailSent(true);
                $invoice->getOrder()->setCustomerNoteNotify(true);
                $invoice->getOrder()->setIsInProcess(true);
                $transactionSave = Mage::getModel('core/resource_transaction')
                    ->addObject($invoice)
                    ->addObject($invoice->getOrder());
                $transactionSave->save();
                $this->setIsOrderChanged(true);
                try {
                    $invoice->sendEmail(true, '');
                } catch (Exception $e) {
                    Mage::logException($e);
                }
            }
        } catch (Exception $ex) {
            Mage::logException($e);
        }

        return $this;
    }

    public function createShipment()
    {
        $order = $this->getOrder();
        if ($shipment = $this->_initShipment($order)) {
            $shipment->register();
            $shipment->setEmailSent(true);
            $shipment->getOrder()->setCustomerNoteNotify(true);

            $shipment->getOrder()->setIsInProcess(true);
            $order = $shipment->getOrder();

            $transactionSave = Mage::getModel('core/resource_transaction')
                ->addObject($shipment)
                ->addObject($order)
                ->save();

            $this->setIsOrderChanged(true);

            try {
                $shipment->sendEmail(true, '');
            } catch (Exception $e) {
                Mage::logException($e);
            }
        }

        return $this;
    }

    protected function _initInvoice($order)
    {
        if (!$order instanceof Mage_Sales_Model_Order) {
            $order = Mage::getModel('sales/order')->load($order);
        }

        if (!$order->getId()) {
            return false;
        }

        if (!$order->canInvoice()) {
            return false;
        }

        $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice(array());
        if (!$invoice->getTotalQty()) {
            Mage::log(Mage::helper('core')->__('Cannot create an invoice without products.'));
            return false;
        }

//        return $invoice;
    }

    protected function _initShipment($order)
    {
        if (!$order instanceof Mage_Sales_Model_Order) {
            $order = Mage::getModel('sales/order')->load($order);
        }

        if (!$order->getId()) {
            return false;
        }

        if ($order->getForcedDoShipmentWithInvoice()) {
            return false;
        }

        if (!$order->canShip()) {
            return false;
        }

//        $shipment = Mage::getModel('sales/service_order', $order)->prepareShipment(array());
//        return $shipment;
    }

    protected function _isCell($phone/*, $country*/)
    {
        $phone = preg_replace("/[^0-9]/", "", $phone);

        //Some customers set the country code instead of a zero. In these cases we simply replace it with a zero
        if(substr($phone, 0, 2) == "46") {
            $phone = substr_replace($phone, "0", 0, 2);
        }

        // if($country == "SE") :
            //A swedish cell phone number is always 10 digits and starts with 07
            if(strlen($phone) == 10 && substr($phone, 0, 2) == "07") {
                return true;
            } else {
                return false;
            }
        /*
        elseif ($country == "DK") :
            //A danish cell phone number starts with 20-31, 40-42, 50-53 or 60-61
            if(substr($phone, 0, 1) == "2" || substr($phone, 0, 2) == "30" || substr($phone, 0, 2) == "31" || substr($phone, 0, 2) == "40" || substr($phone, 0, 2) == "41" || substr($phone, 0, 2) == "42" || substr($phone, 0, 2) == "50" || substr($phone, 0, 2) == "51" || substr($phone, 0, 2) == "52" || substr($phone, 0, 2) == "53" || substr($phone, 0, 2) == "60" || substr($phone, 0, 2) == "61") {
                return true;

            } else {
                return false;
            }
        endif;
        */
    }

    protected function _getConfig($_path, $storeId){
        return Mage::getStoreConfig($_path, $storeId);
    }

}
