<?php

class Infortis_Ultimo_Helper_Labels extends Mage_Core_Helper_Abstract
{
	/**
	 * Get product labels (HTML)
	 *
	 * @return string
	 */
	public function getLabels($product)
	{
		$html = '';

		$isNew = false;
		if (Mage::getStoreConfig('ultimo/product_labels/new'))
		{	
			$isNew = $this->isNew($product);
		}
		
		$isSale = false;
		if (Mage::getStoreConfig('ultimo/product_labels/sale'))
		{
			$isSale = $this->isOnSale($product);
		}
		
		$isFeatured = false;
		if ($product->getData('featured') == true)
		{
			$isFeatured = true;
			$html .= '<span class="sticker-wrapper top-left"><span class="sticker pre-order">'.$this->__('Favorit').'</span></span>';
		}
		
		$preOrder = false;
		if($product->getData('preordercheck') == true) {
			$preOrder = true;
			$html .= '<span class="sticker-wrapper top-right"><span class="sticker pre-order">'.$this->__('Pre Order').'</span></span>';
		}
		
		if ($isNew == true)
		{
			$html .= '<span class="sticker-wrapper top-left"><span class="sticker new">' . $this->__('New') . '</span></span>';
		}
		
		if ($isSale == true)
		{
			$html .= '<span class="sticker-wrapper top-right"><span class="sticker sale">' . $this->__('Sale') . '</span></span>';
		} 
		
		$cup = "";
		if($product->getData('kupa') != "") {
			$cup = $product->getAttributeText('kupa');
			$cupText = $this->__('-cup');
			$cupTextLong = $this->__(' cup');
			if($cup == "D" || $cup == "E" || $cup == "F" || $cup == "G") {
				if($isNew == false) {
					$html .= '<span class="sticker-wrapper top-left"><span class="sticker cup">'.$this->__($cup . $cupText).'</span></span>';
				} else {
					$html .= '<span class="sticker-wrapper top-left lower"><span class="sticker cup">'.$this->__($cup . $cupText).'</span></span>';
				}
			}
		}
		
		return $html;
	}
	
	/**
	 * Check if "new" label is enabled and if product is marked as "new"
	 *
	 * @return  bool
	 */
	public function isNew($product)
	{
		return $this->_nowIsBetween($product->getData('news_from_date'), $product->getData('news_to_date'));
	}
	
	/**
	 * Check if "sale" label is enabled and if product has special price
	 *
	 * @return  bool
	 */
	public function isOnSale($product)
	{
		$specialPrice = number_format($product->getFinalPrice(), 2);
		$regularPrice = number_format($product->getPrice(), 2);
		$bundleSpecialPrice = false;
		
		if($product->getTypeId() == 'bundle') :		
			$typeInstance = $product->getTypeInstance(true);
			$typeInstance->setStoreFilter($product->getStoreId(), $product);

			$optionCollection = $typeInstance->getOptionsCollection($product);

			$selectionCollection = $typeInstance->getSelectionsCollection(
				$typeInstance->getOptionsIds($product),
				$product
			);

			$options = $optionCollection->appendSelections($selectionCollection, false,
				Mage::helper('catalog/product')->getSkipSaleableCheck()
			);
			$summin = 0;
			$summax = 0;
			foreach ($options as $option){
				$selections  = $option->getSelections();
				$minprice = array();
				$_bundleRegularPrice = array();
				foreach ($selections as $key => $_selection){
					$_bundleRegularPrice[] = $_selection->getPrice(); 
					$_bundleRegularPrice[] = round($_bundleRegularPrice);
					$minprice[] = $_selection->getFinalPrice();
				}
				$summin += min($minprice);
				$summax += max($_bundleRegularPrice);
			}
			if ($summin < $summax)
				$bundleSpecialPrice = true;
		endif;
		if ($specialPrice != $regularPrice)
			return $this->_nowIsBetween($product->getData('special_from_date'), $product->getData('special_to_date'));
		elseif ($bundleSpecialPrice == true)
			return true;
		else
			return false;
	}
	
	protected function _nowIsBetween($fromDate, $toDate)
	{
		if ($fromDate)
		{
			$fromDate = strtotime($fromDate);
			$toDate = strtotime($toDate);
			$now = strtotime(Mage::app()->getLocale()->date()->setTime('00:00:00')->toString(Varien_Date::DATETIME_INTERNAL_FORMAT));
			
			if ($toDate)
			{
				if ($fromDate <= $now && $now <= $toDate)
					return true;
			}
			else
			{
				if ($fromDate <= $now)
					return true;
			}
		}
		
		return false;
	}
}
