<?php
/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Magento to newer
 * versions in the future. If you wish to customize Magento for your
 * needs please refer to http://www.magentocommerce.com for more information.
 *
 * @category    Mage
 * @package     Mage_Adminhtml
 * @copyright   Copyright (c) 2012 Magento Inc. (http://www.magentocommerce.com)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * Adminhtml low stock products report grid block
 *
 * @category   Mage
 * @package    Mage_Adminhtml
 * @author      Magento Core Team <core@magentocommerce.com>
 */
class Mage_Adminhtml_Block_Report_Product_Lowstock_Grid extends Mage_Adminhtml_Block_Widget_Grid
{
//    protected $_saveParametersInSession = true;

    public function __construct()
    {
        parent::__construct();
        $this->setId('gridLowstock');
        $this->setUseAjax(false);
    }

    protected function _prepareCollection()
    {
        if ($this->getRequest()->getParam('website')) {
            $storeIds = Mage::app()->getWebsite($this->getRequest()->getParam('website'))->getStoreIds();
            $storeId = array_pop($storeIds);
        } else if ($this->getRequest()->getParam('group')) {
            $storeIds = Mage::app()->getGroup($this->getRequest()->getParam('group'))->getStoreIds();
            $storeId = array_pop($storeIds);
        } else if ($this->getRequest()->getParam('store')) {
            $storeId = (int)$this->getRequest()->getParam('store');
        } else {
            $storeId = '';
        }

        /** @var $collection Mage_Reports_Model_Resource_Product_Lowstock_Collection  */
        $collection = Mage::getResourceModel('reports/product_lowstock_collection')
            ->addAttributeToSelect('*')
            ->setStoreId($storeId)
            ->filterByIsQtyProductTypes()
            ->joinInventoryItem('qty')
            ->useManageStockFilter($storeId)
            ->useNotifyStockQtyFilter($storeId)
            ->setOrder('qty', Varien_Data_Collection::SORT_ORDER_ASC);
			
		$collection = $collection->addExpressionAttributeToSelect('notify_stock_qty', new Zend_Db_Expr("`lowstock_inventory_item`.`notify_stock_qty`"), array());		
		$collection = $collection->addExpressionAttributeToSelect('order_stock_qty', new Zend_Db_Expr("`lowstock_inventory_item`.`notify_stock_qty` - `lowstock_inventory_item`.`qty`"), array());
				
        if( $storeId ) {
            $collection->addStoreFilter($storeId);
        }

        $this->setCollection($collection);
        return parent::_prepareCollection();
    }

    protected function _getAttributeOptions($attribute_code)
    {
        $attribute = Mage::getModel('eav/config')->getAttribute('catalog_product', $attribute_code);
        $options = array();
        foreach( $attribute->getSource()->getAllOptions(true, true) as $option ) {
            $options[$option['value']] = $option['label'];
        }
        return $options;
    }

    protected function _prepareColumns()
    {
        $this->addColumn('name', array(
            'header'    =>Mage::helper('reports')->__('Product Name'),
            'sortable'  =>false,
            'index'     =>'name'
        ));

        $this->addColumn('sku', array(
            'header'    =>Mage::helper('reports')->__('Product SKU'),
            'sortable'  =>true,
            'index'     =>'sku'
        ));

        /*
        $this->addColumn('ean_code', array(
            'header' => Mage::helper('catalog')->__('EAN'),
            'align' => 'left',
            'width' => '180px',
            'index' => 'ean_code',
            'final_index' =>'ean_code',
            'type' => 'options',
            'options' => $this->_getAttributeOptions('ean_code'),
        ));

        $this->addColumn('script_identifier', array(
            'header' => Mage::helper('catalog')->__('Lagersynk'),
            'align' => 'left',
            'width' => '180px',
            'index' => 'script_identifier',
            'final_index' =>'script_identifier',
            'type' => 'options',
            'options' => $this->_getAttributeOptions('script_identifier'),
        ));
        */

        $this->addColumn('c2c_in_price', array(
            'header'        =>Mage::helper('reports')->__('Inköpspris'),
            'width'         =>'60px',
            'align'         =>'center',
            'sortable'      =>true,
            'filter'        =>'adminhtml/widget_grid_column_filter_range',
            'index'         =>'c2c_in_price',
            'type'          =>'number'
        ));

        $this->addColumn('qty', array(
            'header'    =>Mage::helper('reports')->__('Stock Qty'),
            'width'     =>'60px',
            'align'     =>'center',
            'sortable'  =>true,
            'filter'    =>'adminhtml/widget_grid_column_filter_range',
            'index'     =>'qty',
            'type'      =>'number'
        ));
		
		$this->addColumn('preferred_stock_qty', array(
            'header'    =>Mage::helper('reports')->__('Preferred Qty'),
            'width'     =>'60px',
            'align'     =>'center',
            'sortable'  =>false,
            'index'     =>'notify_stock_qty',
            'type'      =>'number'
        ));
		
		$this->addColumn('order_stock_qty', array(
            'header'    =>Mage::helper('reports')->__('Order Qty'),
            'width'     =>'60px',
            'align'     =>'center',
            'sortable'  =>false,
            'index'     =>'order_stock_qty',
            'type'      =>'number'
			));

/*
        $this->addColumn('nos', array(
            'header' => Mage::helper('reports')->__('NOS'),
            'type' => 'options',
            'index' => 'nos',
            'align' => 'center',
            'sortable' => true,
            'options' => $this->_getAttributeOptions('nos')
        ));
*/

        $this->addColumn('manufacturer', array(
            'header' => Mage::helper('catalog')->__('Manufacturer'),
            'align' => 'left',
            'width' => '180px',
            'index' => 'manufacturer',
            'final_index' =>'manufacturer',
            'type' => 'options',
            'options' => $this->_getAttributeOptions('manufacturer'),
        ));
		
		$this->addColumn('wholesaler', array(
			'header'    =>Mage::helper('reports')->__('Leverant&ouml;r'),
			'type'		=>'string',
			'sortable'  =>true,
			'index'     =>'wholesaler'
		));
		
        $this->addExportType('*/*/exportLowstockCsv', Mage::helper('reports')->__('CSV'));
        $this->addExportType('*/*/exportLowstockExcel', Mage::helper('reports')->__('Excel XML'));

        return parent::_prepareColumns();
    }
}
