<?php

class Dv_Labels_Adminhtml_GenerateController extends Mage_Adminhtml_Controller_Action
{

    /**
     * Initialize order model instance
     *
     * @return Mage_Sales_Model_Order || false
     */
    protected function _initOrder($id)
    {
        $order = Mage::getModel('sales/order')->load($id);

        if (!$order->getId()) {
            $this->_getSession()->addError($this->__('This order no longer exists.'));
            $this->_redirect('*/*/');
            $this->setFlag('', self::FLAG_NO_DISPATCH, true);
            return false;
        }

        return $order;
    }

    /**
     * Labels generation page
     */
    public function viewAction()
    {
        if ($order = $this->_initOrder($this->getRequest()->getParam('order_id'))) {

            Mage::register('sales_order', $order);
            Mage::register('current_order', $order);

            $this->_title($this->__('Generate Xml Labels'));

            $this->loadLayout();
            $this->getLayout()->getBlock('xml_labels_view')
                ->updateBackButtonUrl($this->getRequest()->getParam('come_from'));
            $this->_setActiveMenu('sales/order')
                ->renderLayout();
        } else {
            $this->_forward('noRoute');
        }
    }

    protected function _processSave($data)
    {
        $error = false;
        if ($data['xml_type'] !== Dv_Labels_Model_Source_Xmltypes::XML_TYPE_PACKAGE_ID && !$data['weight']){
            $message = $this->__('Weight is required field!');
            $this->setFlag('', self::FLAG_NO_DISPATCH, true);
            $error = true;
        }

        if ($data['xml_type'] === Dv_Labels_Model_Source_Xmltypes::XML_TYPE_CACHE_ON_DELIVERY_ID && !$data['grand_total'] && !$error){
            $message = $this->__('Total Amount is required field!');
            $this->setFlag('', self::FLAG_NO_DISPATCH, true);
            $error = true;
        }

        $order = $this->_initOrder($data['order_id']);
        if ($order && !$error) {

            Mage::register('sales_order', $order);
            Mage::register('current_order', $order);

            try {

                if ( count($order->getShipmentsCollection()) === 0 ){
                    $message = $this->__('Cannot generate labels. Please, create shipment!');
                    $this->setFlag('', self::FLAG_NO_DISPATCH, true);
                    $error = true;
                }

                $_xmlFile = Mage::getModel('xml_labels/xml')->generateXml($data['xml_type'], $data['weight'], $data['grand_total']);
                if ($_xmlFile){
                    $this->_uploadFilesToFTP($_xmlFile);
                    $this->_getSession()->addSuccess($this->__('Label was generate.'));
                } else {
                    $message = $this->__('Cannot generate labels.');
                    $error = true;
                }
            } catch (Mage_Core_Exception $e) {
                $message = $e->getMessage();
                $error = true;
            } catch (Exception $e) {
                Mage::logException($e);
                $message = $this->__('Cannot generate labels.');
                $error = true;
            }
        }

        if ($error){
            $this->_getSession()->addError($message);
            $this->_redirect('*/*/view', array('order_id' => $data['order_id']));
            return false;
        }
        return $this;

    }

    public function saveAction()
    {
        $data = $this->getRequest()->getPost('labels_data');
        $this->_processSave($data);
        $this->_redirect('adminhtml/sales_order/view', array('order_id' => $data['order_id']));
    }

//    public function saveAndUploadAction()
//    {
//        $data = $this->getRequest()->getPost('labels_data');
//        $this->_processSave($data);
//        $this->_uploadFilesToFTP($this->_xmlFile);
//        $this->_redirect('adminhtml/sales_order/view', array('order_id' => $data['order_id']));
//    }

    public function generateAction()
    {
        if ($order = $this->_initOrder($this->getRequest()->getParam('order_id'))) {

            Mage::register('sales_order', $order);
            Mage::register('current_order', $order);

            try {
                $_xmlFile = Mage::getModel('xml_labels/xml')->generateXml();
                if ($_xmlFile){
                    $this->_uploadFilesToFTP($_xmlFile);
                    $this->_getSession()->addSuccess($this->__('Label was generate.'));
                } else {
                    $this->_getSession()->addError($this->__('Cannot generate labels.'));
                }
            } catch (Exception $e){
                $this->_getSession()->addError($e->getMessage());
            }
        }
        $this->_redirect('adminhtml/sales_order/view', array('order_id' => $this->getRequest()->getParam('order_id')));
    }

    public function massGenerateAction()
    {
        if ($this->getRequest()->isPost()) {
            $countSuccess = 0;
            $countFail = 0;
            $orderIds = $this->getRequest()->getPost('order_ids', array());
            $_xmlFiles = array();
            foreach($orderIds as $id){
                if ($order = $this->_initOrder($id)){
                    try {
                        $_xmlFile = Mage::getModel('xml_labels/xml')
                            ->setOrder($order)
                            ->generateXml();

                        if ($_xmlFile){
                            $_xmlFiles[] = $_xmlFile;
                            $countSuccess++;
                        } else {
                            $countFail++;
                        }
                    } catch (Exception $e){
                        $countFail++;
                        Mage::logException($e);
                    }
                } else {
                    $countFail++;
                }
            }

            if (!empty($_xmlFiles)){
                $this->_uploadFilesToFTP($_xmlFiles);
            }

            if ($countFail) {
                if ($countSuccess) {
                    $this->_getSession()->addError(
                        Mage::helper('xml_labels')->__('%s order(s) cannot be processed', $countFail)
                    );
                } else {
                    $this->_getSession()->addError(
                        Mage::helper('xml_labels')->__('The order(s) cannot be processed')
                    );
                }
            }
            if ($countSuccess) {
                $this->_getSession()->addSuccess(
                    Mage::helper('xml_labels')->__('%s order(s) have been processed.', $countSuccess)
                );
            }
        }
        $this->_redirect('adminhtml/sales_order');
    }

    public function massManuallyGenerateAction()
    {
        if ($this->getRequest()->isPost() && $this->getRequest()->getPost('xml_types')) {
            $orderIds = $this->getRequest()->getPost('order_ids', array());
            $countSuccess = 0;
            $countFail = 0;
            $failShipmentIds = array();
            $xmlType = $this->getRequest()->getPost('xml_types');
            $_xmlFiles = array();
            foreach($orderIds as $id) {
                if ( $order = Mage::getModel('sales/order')->load($id)){
                    if ( !$order->canShip() ){
                        try {
                            $_xmlFile = Mage::getModel('xml_labels/xml')
                                ->setOrder($order)
                                ->generateXml($xmlType);

                            if ($_xmlFile){
                                $_xmlFiles[] = $_xmlFile;
                                $countSuccess++;
                            } else {
                                $countFail++;
                            }

                        } catch(Exception $e) {
                            Mage::logException($e);
                            $countFail++;
                        }
                    } else {
                        $countFail++;
                        $failShipmentIds[] = $order->getIncrementId();
                    }
                } else {
                    $countFail++;
                }
            }

            if (!empty($_xmlFiles)){
                $this->_uploadFilesToFTP($_xmlFiles);
            }

            if ( !empty($failShipmentIds) ){
                $this->_getSession()->addError(
                    Mage::helper('xml_labels')->__('You should create shipment(s) for order(s): %s', implode(', ', $failShipmentIds))
                );
            }
            if ($countFail) {
                if ($countSuccess) {
                    $this->_getSession()->addError(
                        Mage::helper('xml_labels')->__('%s order(s) cannot be processed', $countFail)
                    );
                } else {
                    $this->_getSession()->addError(
                        Mage::helper('xml_labels')->__('The order(s) cannot be processed')
                    );
                }
            }
            if ($countSuccess) {
                $this->_getSession()->addSuccess(
                    Mage::helper('xml_labels')->__('%s order(s) have been processed.', $countSuccess)
                );
            }
        }
        $this->_redirect('adminhtml/sales_order/index');
    }

    protected function _uploadFilesToFTP($file)
    {
        try {
            $ftpModel =  Mage::getModel('xml_labels/ftp');
            if (is_array($file)){
                $ftpModel->uploadFiles($file);
            } else {
                $ftpModel->uploadFile($file);
            }
            $ftpModel->closeFTP();
            $this->_getSession()->addSuccess($this->__('Label(s) was(were) upload to FTP server.'));
        } catch(Mage_Core_Exception $e) {
            if ( $ftpModel ){
                $ftpModel->closeFTP();
            }
            $this->_getSession()->addError($this->__('Cannot upload labels to FTP server.'));
        } catch(Exception $e) {
            if ( $ftpModel ){
                $ftpModel->closeFTP();
            }
            $this->_getSession()->addError($this->__('Cannot upload labels to FTP server.'));
        }

    }

}