<?php

class Dv_Labels_Model_Ftp extends Mage_Core_Model_Abstract
{

    protected $_ftpConnection = null;

    protected function _construct()
    {
        $storeId = $this->getStoreId();
        $this->setHost($this->_getConfig('server', $storeId));
        $this->setPort($this->_getConfig('port', $storeId));
        $this->setUser($this->_getConfig('user', $storeId));
        $this->setPass($this->_getConfig('password', $storeId));
        $this->setDir($this->_getConfig('dir', $storeId));
        $this->_initFTP();
        return $this;
    }

    protected function _initFTP()
    {
        $this->connect();
        if ($this->getDir() && $this->getDir() !== '/'){
            if ( !ftp_chdir($this->_ftpConnection, $this->getDir()) ) {
                Mage::throwException('Target directory is not found.');
            }
        }
        return $this;
    }

    protected function _getConfig($_path, $storeId)
    {
        return Mage::getStoreConfig('xmllabels/ftp/' . $_path, $storeId);
    }

    public function connect()
    {
        $connectionStream = ftp_connect($this->getHost(), $this->getPort());
        if (!$connectionStream){
            Mage::throwException($this->__('Could not connect to server! Please, check FTP host and port'));
        } else if (!ftp_login($connectionStream, $this->getUser(), $this->getPass())) {
            Mage::throwException($this->__('Could not connect to server! Please, check FTP User and Password'));
        }
        $this->_ftpConnection = $connectionStream;
        return $this;
    }

    public function uploadFile($file)
    {
        ftp_pasv($this->_ftpConnection, true);
        $fileName = pathinfo($file, PATHINFO_BASENAME);

        if (!ftp_put($this->_ftpConnection, $fileName, $file, FTP_ASCII)){
            Mage::throwException('Could not upload file to server!');
        }
        return $this;
    }

    public function uploadFiles($files)
    {
        foreach($files as $file){
            $this->uploadFile($file);
        }
        return $this;
    }

    public function closeFTP()
    {
        ftp_close($this->_ftpConnection);
        return $this;
    }

    /**
     * Retrieve order model instance
     *
     * @return Mage_Sales_Model_Order
     */
    public function getStoreId()
    {
        if (!$this->getData('store_id')){
            $storeId = Mage::registry('sales_order') ? Mage::registry('sales_order')->getSoterId() : Mage::app()->getStore()->getStoreId();
            $this->setData('store_id', $storeId);
        }
        return $this->getData('store_id');
    }

}