<?php


class Moogento_Pickpack_Helper_Data extends Mage_Core_Helper_Abstract
{
    protected $_simpleProductTypes = array(
        'simple',
        'virtual'
    );

    public function checkStock($order)
    {
        $inStock = true;

        foreach($order->getAllItems() as $orderItem) {

            $stockItem = Mage::getModel('cataloginventory/stock_item')->loadByProduct($orderItem->getProductId());
            if (!$stockItem->getId()) {
                $inStock = false;
                break;
            }

            if (in_array($orderItem->getProductType(), $this->_simpleProductTypes)) {
                if (!$stockItem->getIsInStock()) {
                    $inStock = false;
                    break;
                }

                if ($stockItem->getManageStock() && ($stockItem->getQty() < 1)) {
                    $inStock = false;
                    break;
                }
            } else {
                if (!$stockItem->getIsInStock()) {
                    $inStock = false;
                    break;
                }
            }
        }

        return $inStock;
    }

    public function isShipEasyInstalled()
    {
        $modules = (array)Mage::getConfig()->getNode('modules')->children();

        if (!isset($modules['Moogento_ShipEasy'])) {
            return false;
        }

        $shipEasyConfig = (array)$modules['Moogento_ShipEasy'];

        if (empty($shipEasyConfig['active']) || ($shipEasyConfig['active']!='true')) {
            return false;
        }

        $requiredVersion  = '0.1.14';
        $installedVersion = $shipEasyConfig['version'];

        if ($installedVersion < $requiredVersion) {
            return false;
        }

        if (!Mage::helper('moogento_shipeasy')) {
            return false;
        }

        return true;
    }

/**
 * Resize Image proportionally and return the resized image url
 *
 * @param string $imageName         name of the image file
 * @param integer|null $width       resize width
 * @param integer|null $height      resize height
 * @param string|null $imagePath    directory path of the image present inside media directory
 * @return string               full url path of the image
 */


public function resizeImage($imageName, $width=NULL, $height=NULL, $imagePath=NULL)
{
    $imagePath = str_replace("/", DS, $imagePath);
    $imagePathFull = Mage::getBaseDir('media') . DS . $imagePath . DS . $imageName;
 
    if($width == NULL && $height == NULL) {
        $width = 100;
        $height = 100;
    }
    $resizePath = $width . 'x' . $height;
    $resizePathFull = Mage::getBaseDir('media') . DS . $imagePath . DS . $resizePath . DS . $imageName;
 
    if (file_exists($imagePathFull) && !file_exists($resizePathFull)) {
        $imageObj = new Varien_Image($imagePathFull);
        $imageObj->constrainOnly(TRUE);
        $imageObj->keepAspectRatio(TRUE);
        $imageObj->resize($width,$height);
        $imageObj->save($resizePathFull);
    }
 
    $imagePath=str_replace(DS, "/", $imagePath);
    return Mage::getBaseUrl("media") . $imagePath . "/" . $resizePath . "/" . $imageName;
}

}