<?php

class Moogento_Pickpack_Model_Sales_Order_Processor_Abstract extends Varien_Object
{
    protected $_configGroupPrefix = '';

    protected $_hiddenOrderFlag = ''; 


    public function getPdf($orderIds, $storeId = 0)
    {
        return $this;
    }

    protected function _getFileName($orderIds)
    {
        return $this;
    }

    protected function _getConfig($key, $storeId = 0)
    {
        return Mage::getStoreConfig("pickpack_options/{$this->_configGroupPrefix}/{$key}", $storeId);
    }

    protected function _getConfigFlag($key, $storeId = 0)
    {
        return Mage::getStoreConfigFlag("pickpack_options/{$this->_configGroupPrefix}/{$key}", $storeId);
    }

    protected function _sendAnEmail($pdf, $fileName, $storeId = 0)
    {
        $sender = $this->_getConfig("auto_processing_sender", $storeId);
        $sendTo = $this->_getConfig("auto_processing_send_to", $storeId);
        $subject = $this->_getConfig("auto_processing_subject", $storeId);
        $body = $this->_getConfig("auto_processing_body", $storeId);

        $mailTemplate = Mage::getModel('core/email_template');
        $mailTemplate->setTemplateSubject($subject);
        $mailTemplate->setSenderName(Mage::getStoreConfig('trans_email/ident_'.$sender.'/name'));
        $mailTemplate->setSenderEmail(Mage::getStoreConfig('trans_email/ident_'.$sender.'/email'));
        $mailTemplate->setTemplateText($body);
        $mailTemplate->getTemplateType(Mage_Core_Model_Email_Template::TYPE_TEXT);
        
        $at = new Zend_Mime_Part($pdf->render());
        $at->type        = 'application/pdf';
        $at->disposition = Zend_Mime::DISPOSITION_ATTACHMENT;
        $at->encoding    = Zend_Mime::ENCODING_BASE64;
        $at->filename    = $fileName;
        $mailTemplate->getMail()->addAttachment($at);

        $mailTemplate->send($sendTo);

        return $this;
    }

    protected function _saveAsFile($pdf, $fileName, $storeId = 0)
    {
        try {
            $io = new Varien_Io_File();
            $io->setAllowCreateFolders(true);
            $path = Mage::getBaseDir() . DS . str_replace('/', DS, $this->_getConfig("auto_processing_export_path", $storeId));
            $io->open(array('path'=>$path));
            $io->write($fileName, $pdf->render());
            $io->close();
        } catch (Exception $e) {
            Mage::logException($e);
        }

        return $this;
    }

    protected function _sendToFtp($pdf, $fileName, $storeId = 0)
    {
        try {
            $io = new Varien_Io_Ftp();
            $io->open(array(
                'host'     => $this->_getConfig("auto_processing_ftp_host", $storeId),
                'user'     => $this->_getConfig("auto_processing_ftp_user", $storeId),
                'password' => $this->_getConfig("auto_processing_ftp_password", $storeId),
                'port'     => $this->_getConfig("auto_processing_ftp_port", $storeId),
            ));
            $io->write($fileName, $pdf->render());
            $io->close();
        } catch (Exception $e) {
            Mage::logException($e);
        }
    }

    protected function _processReadyPdf($pdf, $fileName, $storeId = 0)
    {
        switch ($this->_getConfig("auto_processing", $storeId)) {
            case 1:
                //send as email
                $this->_sendAnEmail($pdf, $fileName, $storeId);
                break;
            case 2:
                //save as file
                $this->_saveAsFile($pdf, $fileName, $storeId);
                break;
            case 3:
                //send to ftp
                $this->_sendToFtp($pdf, $fileName, $storeId);
            default:
                //don't do anything
                break;
        }

        return $this;
    }

    protected function _processBulkPdfs($storePdfs)
    {
        $groups = array();

        foreach($storePdfs as $storeId => $storePdfInfo) {

            $storePdf = $storePdfInfo['pdf'];
            $orderIds = $storePdfInfo['order_ids'];

            if ($this->_getConfigFlag("auto_processing_groupping", $storeId)) {
                $storeConfigHash = $this->_getProcessingConfigHash($storeId);
                if (!isset($groups[$storeConfigHash])) {
                    $groups[$storeConfigHash] = array(
                        'store_id'  => $storeId,
                        'pdf'       => $storePdf,
                        'order_ids' => $orderIds
                    );
                } else {
                    $groupedPdf = $groups[$storeConfigHash]['pdf'];
                    foreach($storePdf->pages as $page) {
                        $groupedPdf->pages[] = $page;
                    }

                    $groups[$storeConfigHash]['order_ids'] = array_unique(array_merge($groups[$storeConfigHash]['order_ids'], $orderIds));
                }
            } else {
                $storeConfigHash = md5('store_' . $storeId);
                $groups[$storeConfigHash] = array(
                    'store_id'  => $storeId,
                    'pdf'       => $storePdf,
                    'order_ids' => $orderIds
                );
            }
        }

        foreach($groups as $groupedData) {
            $storeId      = $groupedData['store_id'];
            $pdf          = $groupedData['pdf'];
            $orderIds     = $groupedData['order_ids'];
            $this->_processReadyPdf($pdf, $this->_getFileName($orderIds), $storeId);
        }
    }

    protected function _getProcessingConfigHash($storeId)
    {
        $hashArray['type'] = $this->_getConfig("auto_processing", $storeId);
        switch($hashArray['type']) {
            case 1:
                $hashArray['send_to'] = $this->_getConfig("auto_processing_send_to", $storeId);
                $hashArray['subject'] = $this->_getConfig("auto_processing_subject", $storeId);
                $hashArray['body']    = $this->_getConfig("auto_processing_body", $storeId);
                $hashArray['sender']  = $this->_getConfig("auto_processing_sender", $storeId);
            break;

            case 2:
                $hashArray['path'] = $this->_getConfig("auto_processing_export_path", $storeId);
            break;

            case 3:
                $hashArray['host']     = $this->_getConfig("auto_processing_ftp_host", $storeId);
                $hashArray['user']     = $this->_getConfig("auto_processing_ftp_user", $storeId);
                $hashArray['password'] = $this->_getConfig("auto_processing_ftp_password", $storeId);
                $hashArray['port']     = $this->_getConfig("auto_processing_ftp_port", $storeId);
            break;

            default:
            break;
        }

        return md5(implode('||', $hashArray));
    }

    public function processOrderStatusUpdate($order)
    {
        $storeId = $order->getStoreId();
        if (!$storeId) {
            $storeId = 0;
        }

        if ($this->_getConfig("auto_processing", $storeId)) {
            if ($this->_getConfig("auto_processing_condition_type", $storeId) == 'status') {

                $triggerStatus = $this->_getConfig("auto_processing_condition_status", $storeId);
                if (strpos($triggerStatus, ',') !== false) {
                    $triggerStatus = explode(',', $triggerStatus);
                } else {
                    $triggerStatus = array($triggerStatus);
                }
                
                if (in_array($order->getStatus(), $triggerStatus)) {
                    $checkStock = $this->_getConfigFlag("auto_processing_required_stock", $storeId);

                    if (
                        (!$checkStock) ||
                        ($checkStock && Mage::helper('pickpack')->checkStock($order))
                    ) {

                        $canProcessOrder = true;

                        if ($this->_getConfigFlag("auto_processing_check", $storeId)) {
                            $canProcessOrder = false;
                            $checkField = $this->_getConfig("auto_processing_check_attribute", $storeId);
                            $checkValue = $this->_getConfig("auto_processing_check_attribute_value", $storeId);
                            $orderValue = null;
                            if ($order->hasData($checkField)) {
                                $orderValue = $order->getData($checkField);
                            } else {
                                $orderValue = $this->_getOrderTargetValue($checkField, $order->getId());
                            }
                            if ($this->_ifValueMatched($orderValue, $checkValue)) {
                                $canProcessOrder = true;
                            }
                        }

                        if ($canProcessOrder) {
                            $pdf = $this->getPdf($order->getId(), $storeId);
                            $this->_processReadyPdf($pdf, $this->_getFileName($order->getId()), $storeId);

                            if ($this->_getConfigFlag("auto_processing_set_flag", $storeId)) {
                                $order->setData($this->_hiddenOrderFlag, 1);
                                Mage::getResourceSingleton('pickpack/sales_order')->updateOrderField(
                                    $order->getId(),
                                    $this->_hiddenOrderFlag,
                                    1
                                );
                            }
                            
                        }
                    }

                }
            }
        }

        return $this;
    }

    protected function _getOrderTargetValue($field, $orderId)
    {
        if (in_array($field, array('custom_attribute', 'custom_attribute2'))) {
            return Mage::getResourceSingleton('pickpack/sales_order')->getOrderGridColumnValue($field, $orderId);
        }
        return Mage::getResourceSingleton('pickpack/sales_order')->getOrderColumnValue($field, $orderId);
    }

    protected function _ifValueMatched($orderValue, $neededValue)
    {
        $result = false;

        if ($neededValue === '0') {
            if ((float)$orderValue == (float)$neededValue) {
                $result = true;
            }
        } else if (empty($neededValue)) {
            if (!$orderValue || empty($orderValue)) {
                $result = true;
            }
        } else {
            if ($orderValue == $neededValue) {
                $result = true;
            }
        }

        return $result;
    }

    public function processBulk($digitDay)
    {
        $stores = Mage::app()->getStores(true);

        $ordersByStores = array();

        foreach($stores as $store) {

            $storeId = $store->getId();

            if ($this->_getConfig("auto_processing", $storeId)) {
                if ($this->_getConfig("auto_processing_condition_type", $storeId) == 'day') {
                    $daysToRun = $this->_getConfig("auto_processing_condition_specific_day", $storeId);
                    if (strpos($daysToRun, $digitDay) !== false) {

                        $triggerStatus = $this->_getConfig("auto_processing_condition_status", $storeId);
                        if (strpos($triggerStatus, ',') !== false) {
                            $triggerStatus = explode(',', $triggerStatus);
                        } else {
                            $triggerStatus = array($triggerStatus);
                        }

                        $orders = Mage::getResourceModel('sales/order_collection')->addFieldToFilter('status', array('in' => $triggerStatus));
                        $orders->addFieldToFilter('store_id', $storeId);

                        if (count($orders)) {
                            $checkStock = $this->_getConfigFlag("auto_processing_required_stock", $storeId);
                            $orderIds = array();
                            foreach($orders as $order) {
                                if (!$checkStock) {
                                    $orderIds[] = $order->getId();
                                } else if (Mage::helper('pickpack')->checkStock($order)) {
                                    $orderIds[] = $order->getId();
                                }
                            }

                            if (count($orderIds)) {

                                if ($this->_getConfigFlag("auto_processing_check", $storeId)) {
                                    $checkField = $this->_getConfig("auto_processing_check_attribute", $storeId);
                                    $checkValue = $this->_getConfig("auto_processing_check_attribute_value", $storeId);

                                    $tmpOrderIds = array();
                                    foreach($orderIds as $_orderId) {
                                        $orderValue = $this->_getOrderTargetValue($checkField, $_orderId);
                                        if ($this->_ifValueMatched($orderValue, $checkValue)) {
                                            $tmpOrderIds[] = $_orderId;
                                        }
                                    }

                                    $orderIds = $tmpOrderIds;

                                    if (!count($orderIds)) {
                                        continue;
                                    }
                                }
                                $ordersByStores[$storeId] = $orderIds;
                            }
                        }
                    }
                }
            }
        }

        foreach($ordersByStores as $storeId => $storeOrderIds) {
            $pdf = $this->getPdf($storeOrderIds, $storeId);
            if ($this->_getConfigFlag("auto_processing_set_flag", $storeId)) {
                Mage::getResourceSingleton('pickpack/sales_order')->updateOrderField(
                    $storeOrderIds,
                    $this->_hiddenOrderFlag,
                    1
                );
            }
            $ordersByStores[$storeId] = array(
                'order_ids' => $storeOrderIds,
                'pdf'       => $pdf
            );
        }

        $this->_processBulkPdfs($ordersByStores);

        return $this;
    }
}